SystemOrganization addCategory: #'SystemEditor-Squeak'!

Notification subclass: #InvalidSystemChange
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Squeak'!

InvalidSystemChange subclass: #IllegalVariableName
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Squeak'!

InvalidSystemChange subclass: #InvalidClassFormat
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Squeak'!

!ClassOrganizer methodsFor: '*systemeditor-squeak' stamp: 'mtf 7/18/2008 16:22'!
edClassComment: aString stamp: aStamp
"Set the comment and stamp directly, with no checks.This works around a silly check in ClassOrganizer>>classComment:stamp: in Croquet. This check raises an error if the subject is either nil or a metaclass. The check does not exist in Squeak 3.8 and above"
	^ super classComment: aString stamp: aStamp! !

Object subclass: #ClassExporter
	instanceVariableNames: 'environment classes oldClasses newClasses'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Squeak'!

!ClassExporter commentStamp: 'mtf 10/28/2008 18:55' prior: 0!
This class does a few things. 
- The second step (becomeForward:)of ClassBuilder>>update:to:
- add each class to its superclass's subclasses list
- add each class to the SystemDictionary Smalltalk!

!ClassExporter class methodsFor: 'instance creation' stamp: 'cwp 11/5/2005 17:11'!
on: anEnvironment
	^ self new setEnvironment: anEnvironment! !

!ClassExporter methodsFor: 'public' stamp: 'cwp 11/5/2005 17:11'!
addClass: aClass
	classes add: aClass! !

!ClassExporter methodsFor: 'public' stamp: 'cwp 11/5/2005 18:46'!
commit
	[classes do: [:ea | self export: ea].
	self forwardIdentities]
		valueUnpreemptively
! !

!ClassExporter methodsFor: 'public' stamp: 'cwp 11/5/2005 18:43'!
environment
	^ environment! !

!ClassExporter methodsFor: 'private' stamp: 'mtf 10/29/2008 13:32'!
export: aClass
	(environment includesKey: aClass name)
		ifTrue: [self replace: aClass]
		ifFalse: [environment at: aClass name put: aClass]! !

!ClassExporter methodsFor: 'private' stamp: 'cwp 11/7/2005 20:25'!
forwardIdentities
	oldClasses := oldClasses asArray.
	newClasses := newClasses asArray.
	oldClasses  elementsForwardIdentityTo: newClasses.
	oldClasses := nil.
	newClasses := nil.! !

!ClassExporter methodsFor: 'private' stamp: 'cwp 11/7/2005 20:27'!
replace: aClass
	oldClasses add: (environment at: aClass name).
	newClasses add: aClass! !

!ClassExporter methodsFor: 'initialize-release' stamp: 'cwp 11/5/2005 18:36'!
setEnvironment: anEnvironment 
	environment := anEnvironment.
	classes := OrderedCollection new.
	oldClasses := OrderedCollection new.
	newClasses := OrderedCollection new.! !

Object subclass: #ClassFormat
	instanceVariableNames: 'instSize instSpec cClassIndex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Squeak'!

!ClassFormat commentStamp: 'mtf 7/7/2008 23:18' prior: 0!
I am an expert on the way the Squeak VM encodes class formats. My instance variable, 'bits', is identical in format to the 'format' instance variable of Behavior. I have methods for interpreting those bits. The class format integer is 18 bits in length. The instSize field is split into two parts for backwards compatibility.

<2 bits=instSize//64> <5 bits=cClass> <4 bits=instSpec> <6 bits=instSize\\64> <1 bit=0>

- instSize: the number of named instance variables
- cClass: the index into the compact class table
- instSpec: values indicating how instances are organized
	0 - no state
	1 - named instance variables only
	2 - variable number of object pointers
	3 - named instance variables, variable number of object pointers
	4 - variable number of weak object pointers
	6 - variable number of words
	8 - variable number of bytes
	12 - compiled method format: variable number of object pointers, variable number of bytes!

!ClassFormat class methodsFor: 'instance creation' stamp: 'cwp 9/26/2004 18:14'!
bytes
	^ self size: 0 spec: self bytesSpec index: 0! !

!ClassFormat class methodsFor: 'storage specifications' stamp: 'cwp 9/26/2004 17:52'!
bytesSpec
	^ 8! !

!ClassFormat class methodsFor: 'storage specifications' stamp: 'mtf 7/3/2008 12:11'!
compiledMethodSpec
	^ 12! !

!ClassFormat class methodsFor: 'instance creation' stamp: 'cwp 9/26/2004 17:41'!
fromBits: bits
	^ self
		size: (((bits bitShift: -10) bitAnd: 192) + ((bits bitShift: -1) bitAnd: 63) - 1)
		spec: ((bits bitShift: -7) bitAnd: 15)
		index: ((bits bitShift: -11) bitAnd: 31)! !

!ClassFormat class methodsFor: 'instance creation' stamp: 'cwp 9/26/2004 18:12'!
indexed
	^ self indexed: 0! !

!ClassFormat class methodsFor: 'instance creation' stamp: 'cwp 9/26/2004 18:03'!
indexed: size
	^ self 
		size: size
		spec: (self indexedSpec: size)
		index: 0! !

!ClassFormat class methodsFor: 'storage specifications' stamp: 'cwp 9/26/2004 17:57'!
indexedSpec
	^ 2! !

!ClassFormat class methodsFor: 'storage specifications' stamp: 'cwp 9/26/2004 18:08'!
indexedSpec: size
	^ size = 0 ifTrue: [self indexedSpec] ifFalse: [self namedAndIndexedSpec]! !

!ClassFormat class methodsFor: 'instance creation' stamp: 'cwp 9/26/2004 17:59'!
named
	^ self named: 0! !

!ClassFormat class methodsFor: 'instance creation' stamp: 'cwp 9/26/2004 18:06'!
named: size
	^ self
		size: size
		spec: self namedSpec
		index: 0! !

!ClassFormat class methodsFor: 'storage specifications' stamp: 'cwp 9/26/2004 17:57'!
namedAndIndexedSpec
	^ 3! !

!ClassFormat class methodsFor: 'storage specifications' stamp: 'cwp 9/26/2004 17:55'!
namedSpec
	^ 1! !

!ClassFormat class methodsFor: 'instance creation' stamp: 'cwp 9/26/2004 18:16'!
noState
	^ self size: 0 spec: self noStateSpec index: 0! !

!ClassFormat class methodsFor: 'storage specifications' stamp: 'cwp 9/26/2004 17:55'!
noStateSpec
	^ 0! !

!ClassFormat class methodsFor: 'instance creation' stamp: 'cwp 9/26/2004 17:41'!
size: instSize spec: instSpec index: cClassIndex
	^ self new setSize: instSize spec: instSpec index: cClassIndex! !

!ClassFormat class methodsFor: 'instance creation' stamp: 'mtf 7/8/2008 12:22'!
size: instSize type: aSymbol index: cClassIndex
	^ self size: instSize spec: (self specFromType: aSymbol size: instSize) index: cClassIndex! !

!ClassFormat class methodsFor: 'storage specifications' stamp: 'mtf 7/3/2008 12:10'!
specFromType: aSymbol size: size
	^ aSymbol caseOf: {
		[#normal] -> [size > 0 ifTrue: [self namedSpec] ifFalse: [self noStateSpec]].
		[#variable] -> [self indexedSpec: size].
		[#bytes] -> [self bytesSpec].
		[#words] -> [self wordsSpec].
		[#weak] -> [self weakSpec].
		[#compiledMethod] -> [self compiledMethodSpec].
	} otherwise: [self error: 'Unknown class type: ', aSymbol]
	! !

!ClassFormat class methodsFor: 'instance creation' stamp: 'cwp 9/26/2004 18:18'!
weak
	^ self size: 0 spec: self weakSpec index: 0! !

!ClassFormat class methodsFor: 'storage specifications' stamp: 'cwp 9/26/2004 17:52'!
weakSpec
	^ 4! !

!ClassFormat class methodsFor: 'instance creation' stamp: 'cwp 9/26/2004 18:17'!
words
	^ self size: 0 spec: self wordsSpec index: 0! !

!ClassFormat class methodsFor: 'storage specifications' stamp: 'cwp 9/26/2004 17:52'!
wordsSpec
	^ 6! !

!ClassFormat methodsFor: 'accessing' stamp: 'cwp 9/26/2004 17:37'!
bits
	"Based on ClassBuilder>>format:variable:words:pointers:weak - ar 7/11/1999 06:39 "

	| bits |
	bits := (instSize+1) // 64. 	
	bits := (bits bitShift: 5) + cClassIndex.
	bits := (bits bitShift: 4) + instSpec.
	bits := (bits bitShift: 6) + ((instSize+1)\\64).  "+1 since prim instSize field includes header"
	bits := (bits bitShift: 1). "This shift plus integer bit lets wordSize work like byteSize"
	^ bits! !

!ClassFormat methodsFor: 'accessing' stamp: 'cwp 9/26/2004 17:36'!
cClassIndex
	^ cClassIndex! !

!ClassFormat methodsFor: 'errors' stamp: 'cwp 11/23/2005 22:37'!
error: aString
	InvalidClassFormat signal: aString! !

!ClassFormat methodsFor: 'converting' stamp: 'cwp 9/26/2004 18:15'!
indexedSubclass: nInstVars
	| newSize |
	self isPointers ifFalse: [self error: 'Can''t make a pointer subclass.'].
	newSize := self instSize + nInstVars.
	newSize > 254 ifTrue: [self error: 'Too many instance variables'].
	
	^ self class indexed: newSize! !

!ClassFormat methodsFor: 'accessing' stamp: 'cwp 9/26/2004 17:37'!
instSize
	^ instSize! !

!ClassFormat methodsFor: 'accessing' stamp: 'cwp 9/26/2004 17:37'!
instSpec
	^ instSpec! !

!ClassFormat methodsFor: 'testing' stamp: 'cwp 9/25/2004 13:47'!
isBits
	"Answer whether the receiver contains just bits (not pointers)."

	^ self instSpec >= 6! !

!ClassFormat methodsFor: 'testing' stamp: 'cwp 9/25/2004 13:48'!
isBytes
	"Answer whether the receiver has 8-bit instance variables."

	^ self instSpec >= 8! !

!ClassFormat methodsFor: 'testing' stamp: 'cwp 9/25/2004 13:48'!
isFixed
	"Answer whether the receiver does not have a variable (indexable) part."

	^self isVariable not! !

!ClassFormat methodsFor: 'testing' stamp: 'cwp 9/25/2004 13:48'!
isPointers
	"Answer whether the receiver contains just pointers (not bits)."

	^self isBits not! !

!ClassFormat methodsFor: 'testing' stamp: 'cwp 9/25/2004 13:48'!
isVariable
	"Answer whether the receiver has indexable variables."

	^ self instSpec >= 2! !

!ClassFormat methodsFor: 'testing' stamp: 'cwp 9/25/2004 13:48'!
isWeak
	"Answer whether the receiver has contains weak references."
	^ self instSpec = 4! !

!ClassFormat methodsFor: 'testing' stamp: 'cwp 9/25/2004 13:49'!
isWords
	"Answer whether the receiver has 32-bit instance variables."

	^ self isBytes not! !

!ClassFormat methodsFor: 'converting' stamp: 'cwp 12/11/2005 10:40'!
namedSubclass: nInstVars 
	| newSize |
	newSize := self instSize + nInstVars.
	newSize > 254 ifTrue: [self error: 'Too many instance variables'].
	^ newSize = 0
		ifTrue: [self class noState]
		ifFalse: [self class named: newSize]! !

!ClassFormat methodsFor: 'initializing' stamp: 'cwp 9/26/2004 17:39'!
setSize: size spec: spec index: index
	instSize := size.
	instSpec := spec.
	cClassIndex := index.! !

!ClassFormat methodsFor: 'accessing' stamp: 'mtf 7/8/2008 12:07'!
typeOfClass
	"Answer a symbol uniquely describing the type of the receiver"
	self instSpec = self class compiledMethodSpec ifTrue:[^#compiledMethod]. "Very special!!"
	self isBytes ifTrue:[^#bytes].
	(self isWords and:[self isPointers not]) ifTrue:[^#words].
	self isWeak ifTrue:[^#weak].
	self isVariable ifTrue:[^#variable].
	^#normal.! !

!ClassFormat methodsFor: 'validating' stamp: 'mtf 7/8/2008 12:19'!
validate
"Raise  an error if I am an invalid format"

	instSize > 254 ifTrue: [self error: 'Too many instance variables (', instSize printString,')'].
	(self isBits and: [instSize > 0]) ifTrue: [self error: 'Only pointer formats can have named variables'].
	! !

!ClassFormat methodsFor: 'validating' stamp: 'mtf 7/8/2008 12:59'!
validateAgainstOld: oldFormat
"Raise an error if a class with my format cannot replace a class with oldFormat. Adapted from ClassBuilder >> validateSubclass:canKeepLayoutFrom:forSubclassFormat:"

	oldFormat ifNil: [^ self]. "This class is new, so anything works"
	(self isWeak and: [oldFormat isVariable]) ifTrue: [^ self].
	self typeOfClass ~~ oldFormat typeOfClass
		ifTrue: [self error: 'The new class format is incompatable with the old']! !

!ClassFormat methodsFor: 'validating' stamp: 'mtf 7/8/2008 12:21'!
validateAgainstSuper: superFormat
"Raise an error if a class with my format cannot inherit from a class with superFormat. Adapted from the various checks in the public ClassBuilder methods"

	self instSize < superFormat instSize ifTrue: [self error: 'A subclass lost instance variables'].
	superFormat isFixed ifTrue: [^ self]. "Any format can inherit from #normal"
	superFormat isVariable ~~ self isVariable ifTrue: [self error: 'Cannot derive a fixed class from a variable one'].
	superFormat isBits ~~ self isBits ifTrue: [self error: 'Bit and Pointer classes cannot inherit from one another'].
	superFormat isBytes ~~ self isBytes ifTrue: [self error: 'Byte and Word classes cannot inherit from one another'].
	! !

!ClassFormat methodsFor: 'converting' stamp: 'cwp 9/26/2004 18:28'!
withInstSize: aSmallInteger
	self isPointers ifFalse: [self error: 'Only pointer formats can have named variables'].
	^ self class size: aSmallInteger spec: self instSpec index: self cClassIndex! !

Object subclass: #MethodEditor
	instanceVariableNames: 'source category stamp requestor log repository'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Squeak'!

!MethodEditor class methodsFor: 'as yet unclassified' stamp: 'cwp 10/25/2005 21:36'!
source: source classified: catString stamp: stamp notifying: requestor logging: aBoolean
	^ self new
		setSource: source
		classified: catString
		stamp: stamp
		notifying: requestor
		log: aBoolean! !

!MethodEditor methodsFor: 'accessing' stamp: 'cwp 1/14/2006 19:03'!
category
	^ category! !

!MethodEditor methodsFor: 'building' stamp: 'mtf 11/18/2008 10:58'!
compileFor: aClassEditor

	^ self
		compileForClass: aClassEditor product
		using: aClassEditor ensuredCompilerClass new! !

!MethodEditor methodsFor: 'building' stamp: 'mtf 11/18/2008 10:56'!
compileForClass: aClass using: aCompiler
[
	| node method |
	node := aCompiler
				compile: source
				in: aClass
				notifying: requestor
				ifFail: nil.
	node encoder requestor: self.
	method := node generate: #(0 0 0 0).
	(method respondsTo: #selector:) "set selector on 3.9 and above"
		ifTrue: [method selector: node selector].
	^ method
] on: SyntaxErrorNotification do: [:ex |
	"In 3.10, the class category needs to be set"
	ex instVarNamed: #category put: aClass category.
	"Let the user fix and install the fixed code into the class's old or temporary MethodDictionary"
	ex outer.
	"Now fetch and use that code instead"
	source := aClass sourceCodeAt: self selector ifAbsent: [^ nil].
	^ aClass compiledMethodAt: self selector
]! !

!MethodEditor methodsFor: 'building' stamp: 'cwp 2/1/2006 20:19'!
compileFrom: aCompiledMethod for: aClassEditor
	| result remote |
	result := self compileFor: aClassEditor.
	^ log 
		ifFalse: [result]
		ifTrue: 
			[remote := self logCompilationFrom: aCompiledMethod for: aClassEditor.
			result setSourcePosition: remote position inFile: remote sourceFileNumber]
! !

!MethodEditor methodsFor: 'category browsing' stamp: 'mtf 9/29/2007 14:04'!
elementName
	"The name this editor will be classified under"
	^ self selector! !

!MethodEditor methodsFor: 'accessing' stamp: 'cwp 3/3/2006 15:27'!
interactive
	^ false! !

!MethodEditor methodsFor: 'building' stamp: 'cwp 2/1/2006 20:05'!
logCompilationFrom: aCompiledMethod for: aClassEditor
	^ aCompiledMethod
		ifNil: [self repository 
				storeMethod: source
				forClass: aClassEditor
				category: category
				stamped: stamp]
		ifNotNil: [self repository
					storeMethod: source
					forClass: aClassEditor
					category: category
					stamped: stamp
					replacing: aCompiledMethod]! !

!MethodEditor methodsFor: 'accessing' stamp: 'cwp 2/1/2006 20:20'!
repository
	^ repository ifNil: [repository := SourceRepository default]! !

!MethodEditor methodsFor: 'accessing' stamp: 'cwp 1/31/2006 22:54'!
repository: anObject
	repository := anObject! !

!MethodEditor methodsFor: 'accessing' stamp: 'cwp 10/25/2005 22:50'!
selector
	^ Parser new parseSelector: source! !

!MethodEditor methodsFor: 'initialize-release' stamp: 'cwp 10/25/2005 22:47'!
setSource: sourceString classified: catString stamp: timestamp notifying: anObject log: aBoolean
	source := sourceString.
	category := catString.
	stamp := timestamp.
	requestor := anObject.
	log := aBoolean ! !

!MethodEditor methodsFor: 'accessing' stamp: 'mtf 11/18/2008 10:10'!
source
	^ source! !

!MethodEditor methodsFor: 'accessing' stamp: 'mtf 11/10/2007 01:07'!
timeStamp
	^ stamp! !

Object subclass: #SourceRepository
	instanceVariableNames: 'files'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Squeak'!

!SourceRepository commentStamp: 'mtf 7/14/2008 11:55' prior: 0!
I have no idea what this class is for. It is extensivily tested in SystemEditor-Tests, but it is actually only used two places in the code:
OrganizationEditor>>writeComment
MethodEditor>>logCompilationFrom:for:

I have not examined to see what it does -- Matthew Fulmer!

!SourceRepository class methodsFor: 'accessing' stamp: 'cwp 12/18/2005 14:59'!
createFakeFileStream
	| stream |
	stream := ReadWriteStream on: (String new: 100).
	stream nextPutAll: 'xxx'.
	stream position: 1.
	^ stream! !

!SourceRepository class methodsFor: 'instance creation' stamp: 'cwp 2/4/2006 12:46'!
default
	^ self forSourceFiles: SourceFiles! !

!SourceRepository class methodsFor: 'instance creation' stamp: 'cwp 12/17/2005 06:27'!
forSourceFiles: anArray
	^ self new setSourceFiles: anArray! !

!SourceRepository class methodsFor: 'instance creation' stamp: 'cwp 12/18/2005 15:00'!
newInternal
	^ self forSourceFiles: (Array 
							with: self createFakeFileStream
							with: self createFakeFileStream).! !

!SourceRepository methodsFor: 'comparing' stamp: 'cwp 2/4/2006 12:44'!
= other
	self class == other class ifFalse: [^ false].
	self sourcesStream == other sourcesStream ifFalse: [^ false].
	self changesStream == other changesStream ifFalse: [^ false].
	^ true! !

!SourceRepository methodsFor: 'creation' stamp: 'cwp 2/4/2006 12:32'!
assureStartupStampLogged
	SmalltalkImage current assureStartupStampLogged.! !

!SourceRepository methodsFor: 'accessing' stamp: 'cwp 12/17/2005 06:35'!
changesIndex
	^ 2! !

!SourceRepository methodsFor: 'accessing' stamp: 'cwp 12/17/2005 06:35'!
changesStream
	^ files at: self changesIndex! !

!SourceRepository methodsFor: 'comparing' stamp: 'cwp 2/4/2006 12:44'!
hash
	^ self sourcesStream identityHash bitXor: self changesStream identityHash! !

!SourceRepository methodsFor: 'initialize-release' stamp: 'cwp 12/17/2005 06:27'!
setSourceFiles: anArray
	files := anArray! !

!SourceRepository methodsFor: 'accessing' stamp: 'cwp 2/4/2006 12:42'!
sourcesIndex
	^ 1! !

!SourceRepository methodsFor: 'accessing' stamp: 'cwp 2/4/2006 12:43'!
sourcesStream
	^ files at: self sourcesIndex! !

!SourceRepository methodsFor: 'creation' stamp: 'cwp 1/29/2006 21:32'!
storeComment: aByteString forClass: aClass category: aByteString3 stamped: aByteString4 replacing: aRemoteString 
	self shouldBeImplemented! !

!SourceRepository methodsFor: 'creation' stamp: 'cwp 12/17/2005 06:47'!
storeComment: commentString forClass: aClass stamped: stampString
	^ self storeComment: commentString forClass: aClass stamped: stampString prior: 0! !

!SourceRepository methodsFor: 'creation' stamp: 'cwp 3/3/2006 01:39'!
storeComment: commentString forClass: aClass stamped: stampString prior: aSourcePointer
	| remote stream |
	self assureStartupStampLogged.
	stream := self changesStream.
	stream
		setToEnd;
		cr;
		nextPut: $!!;
		nextPutAll: aClass name;
		nextPutAll: ' commentStamp: ';
		print: stampString;
		nextPutAll: ' prior: ';
		print: aSourcePointer;
		nextPut: $!!;
		cr.
	remote := RemoteString 
				newString: commentString
				onFileNumber: self changesIndex 
				toFile: stream.
		^ remote! !

!SourceRepository methodsFor: 'creation' stamp: 'cwp 12/18/2005 14:05'!
storeComment: commentString forClass: aClass stamped: stampString replacing: aRemoteString
	^ self 
		storeComment: commentString
		forClass: aClass
		stamped: stampString
		prior: aRemoteString sourcePointer! !

!SourceRepository methodsFor: 'creation' stamp: 'cwp 3/3/2006 01:34'!
storeMethod: sourceString forClass: aClass category: catString stamped: stampString 
	^ self storeMethod: sourceString forClass: aClass category: catString stamped: stampString prior: nil! !

!SourceRepository methodsFor: 'creation' stamp: 'cwp 3/3/2006 01:35'!
storeMethod: sourceString forClass: aClass category: catString stamped: stampString prior: sourcePointer
	| remote stream |
	self assureStartupStampLogged.
	stream := self changesStream.
	stream
		setToEnd;
		cr;
		nextPut: $!!;
		nextPutAll: aClass name;
		nextPutAll: ' methodsFor: ';
		print: catString;
		nextPutAll: ' stamp: ';
		print: stampString.
	sourcePointer ifNotNil: [stream nextPutAll: ' prior: '; print: sourcePointer].
	stream
		nextPut: $!!; cr.
	remote := RemoteString 
				newString: sourceString
				onFileNumber: self changesIndex 
				toFile: stream.
	stream space; nextPut: $!!.
		^ remote! !

!SourceRepository methodsFor: 'creation' stamp: 'cwp 1/29/2006 21:49'!
storeMethod: sourceString forClass: aClass category: catString 
stamped: stampString replacing: aRemoteString
	^ self
		storeMethod: sourceString 
		forClass: aClass 
		category: catString 
		stamped: stampString
		prior: aRemoteString sourcePointer.! !

DictionaryEditor subclass: #MethodDictionaryEditor
	instanceVariableNames: 'classEditor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Squeak'!

!MethodDictionaryEditor class methodsFor: 'instance creation' stamp: 'cwp 11/3/2005 07:30'!
empty
	^ self on: MethodDictionary new! !

!MethodDictionaryEditor class methodsFor: 'instance creation' stamp: 'mtf 9/11/2007 23:31'!
for: aClassEditor 
	^ self new setClassEditor: aClassEditor! !

!MethodDictionaryEditor methodsFor: 'editing' stamp: 'mtf 10/28/2008 23:25'!
add: aModifiedMethod
	classEditor edInvalidateOrganization.
	^ additions at: aModifiedMethod selector put: aModifiedMethod! !

!MethodDictionaryEditor methodsFor: 'accessing' stamp: 'mtf 9/11/2007 23:02'!
additions
	^ additions! !

!MethodDictionaryEditor methodsFor: 'reflecting' stamp: 'mtf 9/12/2007 00:03'!
at: aSymbol
	^ self at: aSymbol ifAbsent: [self error: 'Method not found']! !

!MethodDictionaryEditor methodsFor: 'reflecting' stamp: 'mtf 9/20/2007 14:11'!
at: aSymbol ifAbsent: absentBlock
	(removals includes: aSymbol) ifTrue: [^ absentBlock value].
	additions at: aSymbol ifPresent: [:editor | ^ editor].
	self subject at: aSymbol ifPresent: [:aCompiledMethod | ^ self add: (MethodEditor
		source: (classEditor subject sourceCodeAt: aSymbol)
		classified: (classEditor organization subject categoryOfElement: aSymbol)
		stamp: aCompiledMethod timeStamp
		notifying: nil
		logging: true )].
	^ absentBlock value! !

!MethodDictionaryEditor methodsFor: 'editing' stamp: 'mtf 10/6/2008 12:05'!
buildFor: aClassEditor
	| result old |
	result := MethodDictionary new.
	self subject keysAndValuesDo:
		[:selector :cm |
		(self selectorIsModified: selector)
			ifFalse: [result at: selector put: (self recompile: selector from: cm for: aClassEditor)]].
	additions keysAndValuesDo: 
		[:selector :mm | 
		old := self subject at: selector ifAbsent: [nil]. 
		result at: selector put: (mm compileFrom: old for: aClassEditor)].
	^ result
! !

!MethodDictionaryEditor methodsFor: 'building' stamp: 'cwp 1/14/2006 19:02'!
do: aBlock
	additions do: aBlock! !

!MethodDictionaryEditor methodsFor: 'building' stamp: 'mtf 10/28/2008 23:28'!
edPrepareMigration: txn for: aClassEditor
	txn addMigrator: (ObjectMigrator
		origin: self subject
		destination: (self buildFor: aClassEditor))! !

!MethodDictionaryEditor methodsFor: 'building' stamp: 'cwp 2/1/2006 21:42'!
edRequiresBuild
	^ true! !

!MethodDictionaryEditor methodsFor: 'testing' stamp: 'mtf 9/11/2007 23:27'!
includesSelector: aSymbol 
	(additions includesKey: aSymbol) ifTrue: [^ true].
	(removals includes: aSymbol) ifTrue: [^false].
	(self subject includesKey: aSymbol) ifTrue: [^ true].
	^ false! !

!MethodDictionaryEditor methodsFor: 'building' stamp: 'mtf 8/12/2008 15:15'!
recompile: aSelector from: aCompiledMethod for: aClassEditor
	| trailer node |
	aClassEditor edRequiresRecompile ifFalse: [^ aCompiledMethod].
	trailer := aCompiledMethod trailer.
	node := aClassEditor compilerClass new
			compile: (aCompiledMethod 
						getSourceFor: aSelector 
						in: aClassEditor)
			in: aClassEditor product
			notifying: nil
			ifFail: [self error: 'Compilation failed'].
	node selector == aSelector ifFalse: [self error: 'selector changed!!'].
	^ node generate: trailer! !

!MethodDictionaryEditor methodsFor: 'accessing' stamp: 'mtf 9/11/2007 23:02'!
removals
	^ removals! !

!MethodDictionaryEditor methodsFor: 'editing' stamp: 'mtf 10/28/2008 23:26'!
remove: aSymbol
	classEditor edInvalidateOrganization.
	additions removeKey: aSymbol ifAbsent: [].
	removals add: aSymbol! !

!MethodDictionaryEditor methodsFor: 'testing' stamp: 'cwp 10/26/2005 08:44'!
selectorIsModified: aSelector
	^ (additions includesKey: aSelector) or: [removals includes: aSelector]! !

!MethodDictionaryEditor methodsFor: 'initialization' stamp: 'mtf 9/11/2007 23:29'!
setClassEditor: aClassEditor
	classEditor := aClassEditor.
	additions := IdentityDictionary new.
	removals := IdentitySet new.! !

!MethodDictionaryEditor methodsFor: 'accessing' stamp: 'mtf 9/11/2007 23:54'!
subject
	^ subject ifNil: [classEditor subject 
		ifNil: [subject := MethodDictionary new]
		ifNotNilDo: [:aClass | subject := aClass methodDictionary]]! !

DictionaryEditor subclass: #SystemEditor
	instanceVariableNames: 'organization debug'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Squeak'!

!SystemEditor class methodsFor: 'instance creation' stamp: 'cwp 10/28/2005 08:51'!
new
	^ self on: Smalltalk! !

!SystemEditor class methodsFor: 'instance creation' stamp: 'cwp 10/28/2005 08:51'!
on: anEnvironment
	^ self basicNew setSubject: anEnvironment! !

!SystemEditor methodsFor: 'reflecting' stamp: 'mtf 6/26/2008 11:18'!
associationAt: aSymbol 
	^ self 
		associationAt: aSymbol 
		ifAbsent: [self error: 'key not found']! !

!SystemEditor methodsFor: 'reflecting' stamp: 'mtf 8/14/2008 13:50'!
associationAt: aSymbol ifAbsent: aBlock
 
	self at: aSymbol ifAbsent: [^ aBlock value]. "first create any editors necessary"
	^ additions associationAt: aSymbol
		ifAbsent: [subject associationAt: aSymbol]! !

!SystemEditor methodsFor: 'reflecting' stamp: 'cwp 4/14/2006 16:24'!
at: aSymbol 
	^ self 
		at: aSymbol 
		ifAbsent: [self error: 'key not found']! !

!SystemEditor methodsFor: 'reflecting' stamp: 'mtf 8/14/2008 13:49'!
at: aSymbol ifAbsent: aBlock
	| newEditor |
	(removals includes: aSymbol) ifTrue: [^ aBlock value].
	additions at: aSymbol ifPresent: [:editor | ^ editor].
	subject at: aSymbol ifPresent: [:obj |
		newEditor := AbstractEditor on: obj for: self ifNotHandled: [^ obj].
		^ newEditor edRegisterEditor].
	^ aBlock value! !

!SystemEditor methodsFor: 'reflecting' stamp: 'mtf 6/12/2008 23:20'!
at: key ifPresent: aBlock
	"Lookup the given key in the receiver. If it is present, answer the value of evaluating the given block with the value associated with the key. Otherwise, answer nil."

	| v |
	v := self at: key ifAbsent: [^ nil].
	^ aBlock value: v
! !

!SystemEditor methodsFor: 'reflecting' stamp: 'mtf 6/26/2008 11:21'!
bindingOf: aSymbol
 
	^ self associationAt: aSymbol ifAbsent: [ self error: 'could  not bind ', aSymbol ]! !

!SystemEditor methodsFor: 'reflecting' stamp: 'mtf 10/7/2008 13:11'!
classNamed: className 
	"className is either a class name or a class name followed by '
	class'. Answer the class or metaclass it names"
	| meta baseName baseClass |
	(className endsWith: ' class')
		ifTrue: [meta := true.
			baseName := className copyFrom: 1 to: className size - 6]
		ifFalse: [meta := false.
			baseName := className].
	baseClass := self
				at: baseName asSymbol
				ifAbsent: [^ nil].
	meta
		ifTrue: [^ baseClass class]
		ifFalse: [^ baseClass]! !

!SystemEditor methodsFor: 'accessing' stamp: 'mtf 10/7/2008 22:40'!
classOrEditorFor: aClass
"Answers an editor if the class is being edited, or a class if it is not"

	| editor |
	aClass edIsEditor ifTrue: [^ aClass].
	editor := additions at: aClass theNonMetaClass name ifAbsent: [^ aClass].
	^ aClass isMeta ifTrue: [editor class] ifFalse: [editor]! !

!SystemEditor methodsFor: 'building' stamp: 'mtf 12/11/2008 14:15'!
commit

	self commitWithProgress: false withGC: true! !

!SystemEditor methodsFor: 'building' stamp: 'mtf 12/11/2008 14:14'!
commitWithProgress

	self commitWithProgress: true withGC: true! !

!SystemEditor methodsFor: 'building' stamp: 'mtf 12/11/2008 14:14'!
commitWithProgress: showProgress withGC: gc

	| migration export |
	self validate.

	self edExpandEditors.

	migration := MigrationTransaction new.
	self edPrepareMigration: migration withProgress: showProgress.

	export := ClassExporter on: subject.
	self edPrepareExport: export withProgress: showProgress.

	showProgress ifTrue: [(1 to: 2) do: [:ea | ] displayingProgress: 'Committing...'].
	[migration commit.
	export commit.
	self edPostLoad.
	self edCommitRemovals.
	self edRecategorize.
	gc ifTrue: [
		"See ClassBuilder>>update:to: for an explanation of why the
		garbageCollect is necessary. It is optional because some tools,
		like Monticello, already need to do a gc after their own cleanup,
		and the gc's can be merged for efficiency"
		Smalltalk garbageCollect].
	] valueUnpreemptively! !

!SystemEditor methodsFor: 'debugging' stamp: 'mtf 11/26/2008 15:09'!
debug
"see MetaclassEditor>>isDebuggingAsEditor"

	^ debug ifNil: [true]! !

!SystemEditor methodsFor: 'debugging' stamp: 'mtf 9/18/2008 10:44'!
debug: aBoolean
"see MetaclassEditor>>isDebuggingAsEditor"

	debug := aBoolean! !

!SystemEditor methodsFor: 'accessing' stamp: 'mtf 10/13/2008 21:15'!
doItHost
"Answer an object which lets methods evaluated against it execute in my context:

Compiler evaluate: 'Array' for: SystemEditor new doItHost logged: false
"

	^ (self at: #UndefinedObject) new! !

!SystemEditor methodsFor: 'building' stamp: 'cwp 11/6/2005 23:24'!
edClassAt: aSymbol ifAbsent: aBlock
	^ subject at: aSymbol ifAbsent: aBlock! !

!SystemEditor methodsFor: 'building' stamp: 'cwp 12/3/2005 21:26'!
edCommitRemovals
	removals do: [:ea | self edRemove: ea]! !

!SystemEditor methodsFor: 'building' stamp: 'mtf 6/11/2008 20:49'!
edEditorFor: aClass
	| name editor |
	aClass edIsEditor ifTrue: [^ aClass].
	name := aClass theNonMetaClass name.
	editor := self at: name.
	^ aClass isMeta ifFalse: [editor] ifTrue: [editor class].! !

!SystemEditor methodsFor: 'building' stamp: 'mtf 10/30/2008 13:31'!
edExpandEditors
	| queue editor remembered |
	remembered := IdentitySet new.
	queue := additions values asOrderedCollection.
	[queue isEmpty] whileFalse: [editor := queue removeFirst.
		(remembered includes: editor) ifFalse: [
			remembered add: editor.
			editor edDependentsDo: [:ea | queue add: ea]]]! !

!SystemEditor methodsFor: 'testing' stamp: 'cwp 11/20/2005 16:29'!
edIsEditor
	^ true! !

!SystemEditor methodsFor: 'building' stamp: 'mtf 10/28/2008 19:05'!
edPostLoad
	additions do: [:ea | ea edPostLoad]! !

!SystemEditor methodsFor: 'building' stamp: 'mtf 12/11/2008 14:13'!
edPrepareExport: exporter

	self edPrepareExport: exporter withProgress: false! !

!SystemEditor methodsFor: 'building' stamp: 'mtf 12/11/2008 14:12'!
edPrepareExport: exporter withProgress: showProgress

	showProgress
		ifFalse:	[additions do: [:ea | ea edPrepareExport: exporter]]
		ifTrue:	[additions do: [:ea | ea edPrepareExport: exporter] displayingProgress: 'Preparing Classes...']! !

!SystemEditor methodsFor: 'building' stamp: 'mtf 12/11/2008 14:13'!
edPrepareMigration: txn

	self edPrepareMigration: txn withProgress: false! !

!SystemEditor methodsFor: 'building' stamp: 'mtf 12/11/2008 13:59'!
edPrepareMigration: txn withProgress: showProgress

	showProgress
		ifFalse:	[additions do: [:ea | ea edPrepareMigration: txn]]
		ifTrue:	[additions do: [:ea | ea edPrepareMigration: txn] displayingProgress: 'Preparing Transaction...']! !

!SystemEditor methodsFor: 'building' stamp: 'mtf 8/14/2008 13:48'!
edRecategorize
	additions do: 
		[:ea | 
		ea edCategory ifNotNilDo:
			[:cat | 
			SystemOrganization 
				classify: ea name
				under: cat]]! !

!SystemEditor methodsFor: 'building' stamp: 'mtf 8/14/2008 13:50'!
edRegisterEditor: anEditor 
	additions at: anEditor name put: anEditor! !

!SystemEditor methodsFor: 'building' stamp: 'cwp 12/3/2005 21:27'!
edRemove: aSymbol
	| object |
	object := subject at: aSymbol ifAbsent: [^ self].
	object isBehavior
		ifTrue: [object removeFromSystem]
		ifFalse: [subject removeKey: aSymbol]! !

!SystemEditor methodsFor: 'editing' stamp: 'mtf 8/14/2008 13:50'!
edRemoveClassNamed: aSymbol
	additions removeKey: aSymbol ifAbsent: [].
	removals add: aSymbol! !

!SystemEditor methodsFor: 'building' stamp: 'cwp 11/3/2005 07:53'!
edResolve: aSymbol
	^ subject at: aSymbol! !

!SystemEditor methodsFor: 'building' stamp: 'gk 9/5/2007 00:02'!
edResolve: aSymbol ifAbsent: aBlock
	^ subject at: aSymbol ifAbsent: aBlock! !

!SystemEditor methodsFor: 'building' stamp: 'mtf 9/18/2008 12:06'!
edSubclassesOf: anEditor
"Answers all subclasses of the given ClassEditor, as editors"

	^ (self edSubclassesOrEditorsOf: anEditor) collect: [:ea | self edEditorFor: ea]! !

!SystemEditor methodsFor: 'building' stamp: 'mtf 10/13/2008 23:00'!
edSubclassesOf: anEditor do: aBlock
"Answers all subclasses of the given ClassEditor, as editors"

	self edSubclassesOrEditorsOf: anEditor do: [:ea | aBlock value: (self edEditorFor: ea)]! !

!SystemEditor methodsFor: 'building' stamp: 'mtf 10/15/2008 18:25'!
edSubclassesOrEditorsOf: anEditor
"Answer all subclasses of the given ClassEditor, as either an editor (if it has changed), or as a class (if it has not changed). Does not add any items to my additions list"

	| subclasses subeditors |
	subclasses := anEditor subject
		ifNil: [Set new]
		ifNotNilDo: [ :class | class subclasses asSet].
	subeditors := additions select: [ :ea |
		subclasses remove: ea subject ifAbsent: [].
		ea edIsBehavior and: [ea superclassOrEditor == anEditor]].
	removals do: [ :removedKey | subclasses remove: (subject at: removedKey) ifAbsent: []].
	^ subclasses asArray, subeditors asArray! !

!SystemEditor methodsFor: 'building' stamp: 'mtf 11/6/2008 22:39'!
edSubclassesOrEditorsOf: anEditor do: aBlock
"Evaluate aBlock for each subclasses of the given ClassEditor, as either an editor (if it has changed), or as a class (if it has not changed). Does not add any items to my additions list"

	| subclasses |
	subclasses := anEditor subject
		ifNil: [Set new]
		ifNotNilDo: [ :class | class subclasses asSet].
	additions do: [ :ea |
		subclasses remove: ea subject ifAbsent: [].
		((ea isKindOf: ClassDescriptionEditor)
			and: [ea superclassOrEditor == anEditor])
				ifTrue: [aBlock value: ea]].
	removals do: [ :removedKey | subclasses remove: (subject at: removedKey) ifAbsent: []].
	subclasses do: aBlock! !

!SystemEditor methodsFor: 'accessing' stamp: 'mtf 8/14/2008 13:49'!
editors
	^ additions! !

!SystemEditor methodsFor: 'reflecting' stamp: 'mtf 8/14/2008 13:49'!
hasBindingThatBeginsWith: aString
	"Answer true if the receiver has a key that begins with aString, false otherwise"
	
	additions keysDo:[:each | 
		((each beginsWith: aString)
			and: [(removals includes: each) not]) ifTrue:[^true]].
	subject keysDo:[:each | 
		((each beginsWith: aString)
			and: [(removals includes: each) not]) ifTrue:[^true]].
	^false! !

!SystemEditor methodsFor: 'reflecting' stamp: 'mtf 8/14/2008 13:50'!
includesKey: aSymbol 
	(removals includes: aSymbol) ifTrue: [^ false].
	(additions includesKey: aSymbol) ifTrue:  [^ true].
	subject ifNil: [^ false].
	(subject includesKey: aSymbol) ifTrue: [^ true].
	^ false! !

!SystemEditor methodsFor: 'reflecting' stamp: 'mtf 8/14/2008 13:46'!
keyAtIdentityValue: anObject ifAbsent: aBlock
	^ additions 
		keyAtIdentityValue: anObject
		ifAbsent: [subject 
					keyAtIdentityValue: anObject 
					ifAbsent: aBlock]! !

!SystemEditor methodsFor: 'accessing' stamp: 'mtf 9/11/2007 22:47'!
organization
	^ organization ifNil: [organization := SystemOrganizationEditor for: self]! !

!SystemEditor methodsFor: 'accessing' stamp: 'mtf 9/11/2007 20:51'!
removals
	^ removals! !

!SystemEditor methodsFor: 'reflecting' stamp: 'mtf 9/4/2007 15:52'!
renameClass: aClassEditor from: aSymbol 
	self edRemoveClassNamed: aSymbol.
	aClassEditor edRegisterEditor! !

!SystemEditor methodsFor: 'initialize-release' stamp: 'mtf 8/14/2008 13:44'!
setSubject: anEnvironment 
	subject := anEnvironment.
	additions := IdentityDictionary new.
	removals := Set new.! !

!SystemEditor methodsFor: 'accessing' stamp: 'cwp 10/28/2005 08:49'!
subject
	^ subject! !

!SystemEditor methodsFor: 'building' stamp: 'mtf 8/14/2008 13:44'!
validate
	additions do: [:ea | ea validate]! !

!Behavior methodsFor: '*systemeditor-squeak' stamp: 'mtf 11/26/2008 15:39'!
edAllInstVarNames
	^ self allInstVarNames! !

!Behavior methodsFor: '*systemeditor-squeak' stamp: 'mtf 11/26/2008 15:39'!
edInstSize
	^ self instSize! !

!Behavior methodsFor: '*systemeditor-squeak' stamp: 'mtf 11/26/2008 15:40'!
edInstVarNames
	^ self instVarNames! !

AbstractEditor subclass: #CategorizerEditor
	instanceVariableNames: 'subject renamedCategories categories'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Squeak'!

!CategorizerEditor commentStamp: '<historical>' prior: 0!
An abstract superclass for editors of Categorizers. Implements the public interface of a Categorizer. I maintain the list of added, removed, or reordered categories, but it is up to subclasses to remember which elements were added, removed, or recategorized.

subject (Categorizer) - The Categorizer I am editing
categories (OrderedCollection) - The ordered list of categories
renamedCategories (Dictionary) - A dictionary mapping old category names to new names!

!CategorizerEditor methodsFor: 'reflecting - accessing' stamp: 'mtf 9/11/2007 19:10'!
addCategory: newCategory
	^ self addCategory: newCategory before: nil! !

!CategorizerEditor methodsFor: 'reflecting - accessing' stamp: 'mtf 10/6/2008 19:13'!
addCategory: catString before: nextCategory
	"I don't support ordering"
	| newCategory |
	newCategory := catString asSymbol.
	(self categories includes: newCategory) ifTrue: [^ self].
	categories add: newCategory before: nextCategory! !

!CategorizerEditor methodsFor: 'reflecting - accessing' stamp: 'mtf 9/12/2007 10:56'!
allMethodSelectors
	"give a list of all method selectors."

	^ self elementArray copy sort! !

!CategorizerEditor methodsFor: 'reflecting - accessing' stamp: 'mtf 10/6/2008 21:15'!
categories

	^ categories! !

!CategorizerEditor methodsFor: 'reflecting - accessing' stamp: 'mtf 10/6/2008 21:16'!
categories: anArray

	categories := anArray! !

!CategorizerEditor methodsFor: 'accessing changed elements' stamp: 'mtf 10/6/2008 15:38'!
categoryOfChange: element
"Answers the category of a new or recategorized element"

	self changedElementsAndCategoriesDo: [:anElement :aCategory |
		element = anElement ifTrue: [^ aCategory]].
	^ nil! !

!CategorizerEditor methodsFor: 'reflecting - accessing' stamp: 'mtf 1/23/2008 16:55'!
categoryOfElement: element
	(self removedElements includes: element) ifTrue: [^ nil].
	(self categoryOfChange: element) ifNotNilDo: [:cat | ^ cat].
	self subject ifNil: [^ nil].
	^ self newCategoryFor: (self subject categoryOfElement: element)
! !

!CategorizerEditor methodsFor: 'accessing changed elements' stamp: 'mtf 10/6/2008 15:40'!
changedCategories
"Answer the set of categories whose elements have changed"

	^ (self changedElementsAndCategoriesCollect:
		[:anElement :aCategory | aCategory]) asSet
! !

!CategorizerEditor methodsFor: 'accessing changed elements' stamp: 'mtf 10/6/2008 15:41'!
changedElements
"Answers the list of elements that have been added or recategorized"

	^ self changedElementsAndCategoriesCollect: [:anElement :aCategory | anElement]! !

!CategorizerEditor methodsFor: 'accessing changed elements' stamp: 'mtf 10/6/2008 15:45'!
changedElementsAndCategoriesCollect: aBlock
"Evaluates aBlock for each added or recategorized elements, and its category. Answers an OrderedCollection of the results"

	| newCollection |
	newCollection := OrderedCollection new.
	self changedElementsAndCategoriesDo: [:anElement :aCategory |
		newCollection add: (aBlock value: anElement value: aCategory)].
	^ newCollection! !

!CategorizerEditor methodsFor: 'accessing changed elements' stamp: 'mtf 10/6/2008 15:45'!
changedElementsAndCategoriesDo: aBlock
"Evaluates aBlock for each added or recategorized elements, and its category"

	self subclassResponsibility! !

!CategorizerEditor methodsFor: 'accessing changed elements' stamp: 'mtf 10/6/2008 15:52'!
changedElementsAndCategoriesSelect: selectBlock thenCollect: collectBlock
"Evaluates collectBlock for each added or recategorized element and its category for which selectBlock answers true. Answers an OrderedCollection of the results"

	| newCollection |
	newCollection := OrderedCollection new.
	self changedElementsAndCategoriesDo: [:anElement :aCategory |
		(selectBlock value: anElement value: aCategory) ifTrue: [newCollection
			add: (collectBlock value: anElement value: aCategory)]].
	^ newCollection! !

!CategorizerEditor methodsFor: 'accessing changed elements' stamp: 'mtf 10/6/2008 15:46'!
changesInCategory: category
"Answers a list of new or recategorized elements in category"

	^ self changedElementsAndCategoriesSelect:
			[:anElement :aCategory | category = aCategory]
		thenCollect: [:anElement :aCategory | anElement]! !

!CategorizerEditor methodsFor: 'reflecting - accessing' stamp: 'mtf 9/11/2007 19:44'!
classify: element under: heading 
	self classify: element under: heading suppressIfDefault: true! !

!CategorizerEditor methodsFor: 'reflecting - accessing' stamp: 'mtf 10/7/2008 10:18'!
classify: element under: heading suppressIfDefault: aBoolean
	| category |
	((heading = Categorizer nullCategory) or: [heading == nil])
		ifTrue: [category := Categorizer default]
		ifFalse: [category := heading asSymbol].
	self addCategory: category.
	((self categories includes: Categorizer default)
		and: [(self listAtCategoryNamed: Categorizer default) size = 0])
			ifTrue: [self removeCategory: Categorizer default].
! !

!CategorizerEditor methodsFor: 'reflecting - accessing' stamp: 'mtf 9/11/2007 19:45'!
classifyAll: aCollection under: heading

	aCollection do:
		[:element | self classify: element under: heading]! !

!CategorizerEditor methodsFor: 'building' stamp: 'mtf 10/26/2008 09:17'!
edBuild
	| result processed newCat |
	result := self productClass defaultList: Array new.
	categories ifNotEmpty: [result categories: categories].
	self changedElementsAndCategoriesDo: [:ea :cat |
		result classify: ea under: cat].
	self subject ifNil: [^ result].
	processed := IdentitySet new 
		addAll: result elementArray;
		addAll: self removedElements;
		yourself.
	self subject categories do: [:oldCat |
		newCat := self newCategoryFor: oldCat.
		(subject listAtCategoryNamed: oldCat) do: [:ea |
			(processed includes: ea) ifFalse: [
				result classify: ea under: newCat]]].
	^ result! !

!CategorizerEditor methodsFor: 'building' stamp: 'mtf 10/6/2008 22:03'!
edPrepareMigration: txn
	self edRequiresBuild
		ifTrue: [txn addMigrator: (ObjectMigrator 
									origin: self subject 
									destination: self edBuild)]! !

!CategorizerEditor methodsFor: 'building' stamp: 'mtf 10/6/2008 14:58'!
edRequiresBuild
	^ self hasChangedElements or: [self removedElements notEmpty]! !

!CategorizerEditor methodsFor: 'reflecting - private' stamp: 'mtf 9/12/2007 10:55'!
elementArray
	^ self categories gather: [:category | self listAtCategoryNamed: category]! !

!CategorizerEditor methodsFor: 'accessing changed elements' stamp: 'mtf 10/6/2008 14:56'!
hasChangedElements
	self changedElementsAndCategoriesDo: [:element :category | ^ true].
	^ false! !

!CategorizerEditor methodsFor: 'reflecting - accessing' stamp: 'mtf 10/7/2008 10:08'!
listAtCategoryNamed: catName

	| category list |
	category := catName asSymbol.
	(self categories includes: category) ifFalse: [^ nil].
	list := self subject ifNotNil: [self subject listAtCategoryNamed: category].
	list ifNil: [list := OrderedCollection new].
	list := list asOrderedCollection.
	self removedElements do: [:ea | list removeAll: ea].
	self changedElementsAndCategoriesDo: [:ea :cat |
		list removeAll: ea. cat = category ifTrue: [list add: ea]].
	^ list
! !

!CategorizerEditor methodsFor: 'reflecting - accessing' stamp: 'mtf 9/11/2007 19:59'!
listAtCategoryNumber: anInteger
	^ self listAtCategoryNamed: (self categories at: anInteger)! !

!CategorizerEditor methodsFor: 'as yet unclassified' stamp: 'mtf 9/11/2007 19:14'!
newCategoryFor: oldCategory
	^ renamedCategories at: oldCategory ifAbsent: [oldCategory]! !

!CategorizerEditor methodsFor: 'as yet unclassified' stamp: 'mtf 9/11/2007 19:14'!
oldCategoryFor: newCategory
	^ renamedCategories keyAtValue: newCategory ifAbsent: [newCategory]! !

!CategorizerEditor methodsFor: 'reflecting - printing' stamp: 'mtf 9/11/2007 19:45'!
printOn: aStream
	self categories do: [:category |
		aStream nextPutAll: '('''; nextPutAll: category; nextPut: $'.
		(self listAtCategoryNamed: category) do: [:element |
			aStream space; nextPutAll: element].
		aStream nextPut: $); cr]! !

!CategorizerEditor methodsFor: 'building' stamp: 'mtf 10/7/2008 14:55'!
productClass
	^ Categorizer! !

!CategorizerEditor methodsFor: 'reflecting - accessing' stamp: 'mtf 10/7/2008 10:19'!
removeCategory: catString
	| category |
	category := catString asSymbol.
	(self categories includes: category) ifFalse: [^ self].
	(self listAtCategoryNamed: category)
		ifNotEmpty: [^self error: 'cannot remove non-empty category'].
	categories remove: category ifAbsent: [].
	renamedCategories keysAndValuesRemove: [:key :value | value == category].
! !

!CategorizerEditor methodsFor: 'reflecting - accessing' stamp: 'mtf 10/7/2008 10:24'!
removeElement: element
	self subclassResponsibility! !

!CategorizerEditor methodsFor: 'accessing changed elements' stamp: 'mtf 10/6/2008 15:47'!
removedElements
"Answers a list of elements that have been removed"

	^ Array new! !

!CategorizerEditor methodsFor: 'reflecting - accessing' stamp: 'mtf 10/7/2008 10:24'!
renameCategory: oldCatString toBe: newCatString
	| oldCategory newCategory originalCategory |
	newCategory := newCatString asSymbol.
	oldCategory := oldCatString asSymbol.
	originalCategory := self oldCategoryFor: oldCategory.
	(self categories includes: newCategory) ifTrue: [^ self].
	(self categories includes: oldCategory) ifFalse: [^ self].
	categories replaceAll: oldCategory with: newCategory.
	renamedCategories at: originalCategory put: newCategory.! !

!CategorizerEditor methodsFor: 'initialize-release' stamp: 'mtf 10/7/2008 10:11'!
setSubject: aCategorizer
	subject := aCategorizer.
	renamedCategories := Dictionary new.
	categories := OrderedCollection new.! !

!CategorizerEditor methodsFor: 'accessing' stamp: 'mtf 9/11/2007 19:07'!
subject
	^ subject! !

CategorizerEditor subclass: #OrganizationEditor
	instanceVariableNames: 'classEditor isDirty comment stamp remote repository'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Squeak'!

!OrganizationEditor commentStamp: 'mtf 7/14/2008 11:28' prior: 0!
My instances are editors for ClassOrganizer instances!

!OrganizationEditor class methodsFor: 'instance creation' stamp: 'cwp 12/18/2005 14:47'!
for: aClassEditor
	^ self for: aClassEditor repository: SourceRepository default! !

!OrganizationEditor class methodsFor: 'instance creation' stamp: 'cwp 12/18/2005 14:47'!
for: aClassEditor repository: aSourceRepository
	^ self new setClassEditor: aClassEditor repository: aSourceRepository! !

!OrganizationEditor class methodsFor: 'instance creation' stamp: 'cwp 12/15/2005 22:20'!
on: anOrganizer
	^ self for: anOrganizer subject! !

!OrganizationEditor methodsFor: 'accessing' stamp: 'mtf 1/24/2008 20:57'!
changedElementsAndCategoriesDo: aBlock
	classEditor methods additions keysAndValuesDo: [:key :editor |
		aBlock value: key value: editor category]! !

!OrganizationEditor methodsFor: 'SystemEditor-Editors' stamp: 'mtf 9/11/2007 17:06'!
classComment
	^comment ifNil: [self subject ifNotNil: [self subject classComment]]! !

!OrganizationEditor methodsFor: 'reflecting' stamp: 'cwp 2/1/2006 22:49'!
classComment: aStringOrText stamp: aString
	comment := aStringOrText.
	stamp := aString.
	isDirty := true
	! !

!OrganizationEditor methodsFor: 'SystemEditor-Editors' stamp: 'mtf 9/11/2007 17:06'!
commentRemoteStr
	"If we are dirty the comment has been explicitly set and if
	it was set to something else than nil we lazily create the remote String."

	(isDirty and: [comment notNil] and: [remote isNil]) ifTrue: [self writeComment].
	^ remote ifNil: [self subject ifNotNil: [self subject commentRemoteStr]]! !

!OrganizationEditor methodsFor: 'reflecting' stamp: 'mtf 9/11/2007 17:07'!
commentStamp
	^stamp ifNil: [self subject ifNotNil: [self subject commentStamp]]! !

!OrganizationEditor methodsFor: 'building' stamp: 'mtf 10/6/2008 22:05'!
edBuild
	| result |
	result := super edBuild.
	result
		edClassComment: self commentRemoteStr
		stamp: self commentStamp.
	^ result! !

!OrganizationEditor methodsFor: 'building' stamp: 'mtf 10/6/2008 22:08'!
edRequiresBuild
	^ isDirty or: [super edRequiresBuild]! !

!OrganizationEditor methodsFor: 'building' stamp: 'mtf 10/7/2008 14:56'!
productClass
	^ ClassOrganizer! !

!OrganizationEditor methodsFor: 'accessing' stamp: 'mtf 1/25/2008 19:45'!
removedElements
	^ classEditor methods removals! !

!OrganizationEditor methodsFor: 'initialize-release' stamp: 'mtf 9/11/2007 17:13'!
setClassEditor: aClassEditor repository: aSourceRepository
	isDirty := false.
	classEditor := aClassEditor.
	repository := aSourceRepository.
	self setSubject: nil.! !

!OrganizationEditor methodsFor: 'accessing' stamp: 'mtf 9/11/2007 17:17'!
subject
	^ subject ifNil: [classEditor subject ifNotNilDo: [:aClass | subject := aClass organization]]! !

!OrganizationEditor methodsFor: 'building' stamp: 'mtf 8/29/2007 14:59'!
writeComment
	| previous subject |
	subject := self subject.
	previous := subject ifNotNil: [subject commentRemoteStr].
	remote := previous 
				ifNil: [repository 
						storeComment: comment
						forClass: classEditor
						stamped: stamp]
				ifNotNil: [repository 
							storeComment: comment
							forClass: classEditor
							stamped: stamp
							replacing: previous].! !

CategorizerEditor subclass: #SystemOrganizationEditor
	instanceVariableNames: 'system'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Squeak'!

!SystemOrganizationEditor class methodsFor: 'as yet unclassified' stamp: 'mtf 9/11/2007 22:08'!
for: anEnvironment
	^ self new setSystem: anEnvironment! !

!SystemOrganizationEditor methodsFor: 'accessing' stamp: 'mtf 1/24/2008 19:04'!
changedElementsAndCategoriesDo: aBlock
	system editors keysAndValuesDo: [:key :editor |
		aBlock value: key value: editor category]! !

!SystemOrganizationEditor methodsFor: 'accessing' stamp: 'mtf 1/24/2008 20:54'!
removedElements
	^ system removals! !

!SystemOrganizationEditor methodsFor: 'as yet unclassified' stamp: 'mtf 9/11/2007 22:09'!
setSystem: anEnvironment
	system := anEnvironment! !

!SystemOrganizationEditor methodsFor: 'accessing' stamp: 'mtf 9/11/2007 22:13'!
subject
	^ subject ifNil: [self setSubject: system subject organization. subject]! !

AbstractEditor subclass: #PureBehaviorEditor
	instanceVariableNames: 'superclass methodDict format subject product system methods organization properties decorators compilerClass'
	classVariableNames: 'ReservedNames'
	poolDictionaries: ''
	category: 'SystemEditor-Squeak'!

!PureBehaviorEditor class methodsFor: 'initialize-release' stamp: 'mtf 10/13/2008 15:33'!
initialize
	| strings |
	strings := #('self' 'super' 'true' 'false' 'nil' 'thisContext').
	ReservedNames := strings, strings collect: [:ea | ea asSymbol].! !

PureBehaviorEditor subclass: #ClassDescriptionEditor
	instanceVariableNames: 'superEditor type instVarNames'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Squeak'!

!ClassDescriptionEditor commentStamp: 'mtf 7/29/2008 21:55' prior: 0!
I am an abstract superclass for both ClassEditor and MetaclassEditor.

I implement  the Behavior protocol: my instances can create instances. This means that my first three variables must remain 'superclass methodDict format', as the VM accesses them positionally, and my superclasses must not define any instance vairables. Only MetaclassEditor makes use of these ivars, but they are defined here to avoid duplicating a lot of ivars and accessing methods between ClassEditor and MetaclassEditor!

!ClassDescriptionEditor methodsFor: 'editing' stamp: 'mtf 8/5/2008 22:59'!
addInstVarName: aString 
	instVarNames ifNil: [instVarNames := self subject instVarNames].
	(instVarNames includes: aString)
		ifFalse: [instVarNames := instVarNames copyWith: aString]! !

!ClassDescriptionEditor methodsFor: 'reflecting' stamp: 'mtf 11/26/2008 15:02'!
allInstVarNames
	^ self edAllInstVarNames! !

!ClassDescriptionEditor methodsFor: 'reflecting' stamp: 'mtf 10/13/2008 22:49'!
allSubclassesDo: aBlock 
	"Evaluate the argument, aBlock, for each of the receiver's subclasses."

	self subclassesDo: 
		[:cl | 
		aBlock value: cl.
		cl allSubclassesDo: aBlock]! !

!ClassDescriptionEditor methodsFor: 'reflecting' stamp: 'mtf 8/6/2008 00:01'!
allSuperclasses
"Answer an OrderedCollection of the receiver's and the receiver's ancestor's superclasses. The first element is the receiver's immediate superclass, followed by its superclass; the last element is Object."

	^ self superclass withAllSuperclasses! !

!ClassDescriptionEditor methodsFor: 'reflecting' stamp: 'mtf 10/13/2008 21:42'!
allSuperclassesDo: aBlock 
"Evaluate aBlock for each of my receiver superclasses."

	self superclass withAllSuperclassesDo: aBlock! !

!ClassDescriptionEditor methodsFor: 'reflecting' stamp: 'mtf 10/13/2008 21:09'!
basicNew
	"Primitive. Answer an instance of the receiver (which is a class) with no 
	indexable variables. Fail if the class is indexable. Essential. See Object 
	documentation whatIsAPrimitive."

	<primitive: 70>
	"space must be low"
	self environment signalLowSpace.
	^ self basicNew  "retry if user proceeds"
! !

!ClassDescriptionEditor methodsFor: 'reflecting' stamp: 'mtf 8/6/2008 00:01'!
browserDefinition: style
	^ self definition! !

!ClassDescriptionEditor methodsFor: 'reflecting' stamp: 'mtf 8/6/2008 00:04'!
classSide
	^self theMetaClass! !

!ClassDescriptionEditor methodsFor: 'reflecting' stamp: 'mtf 8/6/2008 00:15'!
definition
	^ self definitionST80! !

!ClassDescriptionEditor methodsFor: 'reflecting' stamp: 'mtf 8/6/2008 00:15'!
definitionST80: isST80
	^ isST80 ifTrue: [self definitionST80] ifFalse: [self definition]! !

!ClassDescriptionEditor methodsFor: 'accessing' stamp: 'mtf 11/26/2008 15:00'!
edAllInstVarNames
	^ self superclassOrEditor edAllInstVarNames , self edInstVarNames! !

!ClassDescriptionEditor methodsFor: 'building' stamp: 'mtf 11/26/2008 14:38'!
edClassFormat
	^ ClassFormat
		size: self edInstSize
		type: self typeOfClass
		index: self indexIfCompact! !

!ClassDescriptionEditor methodsFor: 'accessing' stamp: 'mtf 11/26/2008 14:33'!
edInstSize
	^ self edInstVarNames size + self superclassOrEditor edInstSize! !

!ClassDescriptionEditor methodsFor: 'accessing' stamp: 'mtf 11/26/2008 14:23'!
edInstVarNames
	^ instVarNames ifNil: [self subject instVarNames]! !

!ClassDescriptionEditor methodsFor: 'building' stamp: 'mtf 10/28/2008 23:42'!
edInvalidateSuperclass
"Note that I require a rebuild due to a superclass change"
	self superclass: self superclass! !

!ClassDescriptionEditor methodsFor: 'testing' stamp: 'mtf 10/15/2008 18:23'!
edIsBehavior
"Answers true if I am an editor for a Behavior"
	^ true! !

!ClassDescriptionEditor methodsFor: 'building' stamp: 'mtf 7/29/2008 08:57'!
edOldFormat
	^ self subject ifNotNil: [ClassFormat fromBits: self subject format]! !

!ClassDescriptionEditor methodsFor: 'building' stamp: 'mtf 11/26/2008 18:38'!
edRequiresRecompile
	instVarNames ifNotNil: [^ true].
	self edSuperclassHasChanged ifTrue: [^ true].
	self decoratorsDo: [:ea | ea edRequiresRecompile ifTrue: [^ true]].
	^ false! !

!ClassDescriptionEditor methodsFor: 'building' stamp: 'mtf 11/26/2008 15:14'!
edSuperFormat
	^ self superclassOrEditor 
		ifNotNilDo: [ :ed | ClassFormat fromBits: ed format ]
		ifNil: [ ClassFormat named ]! !

!ClassDescriptionEditor methodsFor: 'building' stamp: 'mtf 9/26/2008 11:23'!
edSuperclass
"Answer my product's superclass, building it if necessary"

	| classOrEditor |
	classOrEditor :=  self superclassOrEditor.
	^ classOrEditor edIsEditor
		ifTrue: [classOrEditor product]
		ifFalse: [classOrEditor]! !

!ClassDescriptionEditor methodsFor: 'building' stamp: 'mtf 11/26/2008 18:41'!
edSuperclassHasChanged
	^ superEditor ~~ nil! !

!ClassDescriptionEditor methodsFor: 'accessing' stamp: 'mtf 11/18/2008 11:44'!
edUltimateMethodAt: aSelector ifAbsent: aBlock
	^ self edMethodAt: aSelector ifAbsent: [
		self superclass edUltimateMethodAt: aSelector ifAbsent: aBlock]! !

!ClassDescriptionEditor methodsFor: 'editing' stamp: 'mtf 8/5/2008 23:11'!
ensureInstVarName: aString 
	(self allInstVarNames includes: aString)
		ifFalse: [ self addInstVarName: aString].
		 ! !

!ClassDescriptionEditor methodsFor: 'reflecting' stamp: 'mtf 8/6/2008 00:17'!
format
	^ self edClassFormat bits! !

!ClassDescriptionEditor methodsFor: 'reflecting' stamp: 'mtf 8/6/2008 00:19'!
indexIfCompact
	self subject ifNil: [^ 0].
	^ self subject indexIfCompact! !

!ClassDescriptionEditor methodsFor: 'reflecting' stamp: 'mtf 11/26/2008 14:41'!
instSize
	^ self edInstSize! !

!ClassDescriptionEditor methodsFor: 'editing' stamp: 'mtf 11/26/2008 14:27'!
instVarNames
	^ self edInstVarNames! !

!ClassDescriptionEditor methodsFor: 'reflecting' stamp: 'mtf 8/6/2008 00:23'!
instanceSide
	^ self theNonMetaClass! !

!ClassDescriptionEditor methodsFor: 'reflecting' stamp: 'mtf 8/6/2008 00:24'!
instanceVariableNames: instString
	instVarNames := Scanner new scanFieldNames: instString! !

!ClassDescriptionEditor methodsFor: 'reflecting' stamp: 'mtf 11/26/2008 15:00'!
instanceVariablesString
	^ String streamContents: 
		[:stream | 
		self edInstVarNames
			do: [:ea | stream nextPutAll: ea]
			separatedBy: [stream nextPut: $ ]]! !

!ClassDescriptionEditor methodsFor: 'testing' stamp: 'mtf 8/6/2008 00:33'!
isBits
	^ #(bytes words) includes: self typeOfClass! !

!ClassDescriptionEditor methodsFor: 'testing' stamp: 'mtf 8/6/2008 00:33'!
isBytes
	^ self typeOfClass = #bytes! !

!ClassDescriptionEditor methodsFor: 'testing' stamp: 'mtf 8/6/2008 00:34'!
isPointers
	^ #(normal weak variable) includes: self typeOfClass! !

!ClassDescriptionEditor methodsFor: 'testing' stamp: 'mtf 8/6/2008 00:34'!
isVariable
	^ (self typeOfClass = #normal) not! !

!ClassDescriptionEditor methodsFor: 'testing' stamp: 'mtf 8/6/2008 00:34'!
isWeak
	^ self typeOfClass = #weak! !

!ClassDescriptionEditor methodsFor: 'testing' stamp: 'mtf 8/6/2008 00:34'!
isWords
	^ #(words weak variable) includes: self typeOfClass ! !

!ClassDescriptionEditor methodsFor: 'reflecting' stamp: 'mtf 8/6/2008 00:36'!
kindOfSubclass
	^ self typeOfClass caseOf: {
		[#normal] -> [' subclass: '].
		[#words] -> [' variableWordSubclass: '].
		[#variable] -> [' variableSubclass: '].
		[#bytes] -> [' variableByteSubclass: '].
		[#weak] -> [' weakSubclass: ' ].
		[#compiledMethod] -> [' variableByteSubclass: ' ]
	} otherwise: [self error: 'Unrecognized class type']! !

!ClassDescriptionEditor methodsFor: 'debugging' stamp: 'mtf 11/18/2008 10:27'!
lookupSelector: aSelector
	^ superclass lookupSelector: aSelector! !

!ClassDescriptionEditor methodsFor: 'editing' stamp: 'mtf 8/5/2008 23:16'!
rearrangeInstVarNames: associations
	"The associations are name -> pos and cover the subset
	of vars that should be moved. The old just fill in the holes."

	instVarNames := ClassEditor rearrangeVarNames: associations oldNames: instVarNames! !

!ClassDescriptionEditor methodsFor: 'editing' stamp: 'mtf 8/5/2008 23:22'!
removeInstVarName: aString 
	instVarNames ifNil: [instVarNames := self subject instVarNames].
	instVarNames := instVarNames copyWithout: aString! !

!ClassDescriptionEditor methodsFor: 'reflecting' stamp: 'mtf 8/6/2008 00:50'!
superclass
"Answers the editor on my new superclass, creating it if necessary"

	^ self environment edEditorFor: self superclassOrEditor! !

!ClassDescriptionEditor methodsFor: 'editing' stamp: 'mtf 8/5/2008 23:52'!
superclass: aClassOrEditor 
	superEditor := self environment edEditorFor: aClassOrEditor! !

!ClassDescriptionEditor methodsFor: 'accessing' stamp: 'mtf 10/13/2008 23:29'!
superclassOrEditor
"Answer my superclass as an editor if it is being edited, or as a class if it is not. Does not add anything to my SystemEditor"

	superEditor ifNotNil: [^ superEditor].
	self subject superclass ifNil: [^ self realClass classRootEditor on: nil for: self environment].
	^ self environment classOrEditorFor: self subject superclass! !

!ClassDescriptionEditor methodsFor: 'accessing' stamp: 'mtf 8/6/2008 01:01'!
typeOfClass
	type ifNotNil: [^ type].
	self subject ifNotNil: [^ self subject typeOfClass].
	^ self superclassOrEditor typeOfClass.! !

!ClassDescriptionEditor methodsFor: 'accessing' stamp: 'mtf 8/5/2008 23:52'!
typeOfClass: aSymbol
	type := aSymbol! !

!ClassDescriptionEditor methodsFor: 'reflecting' stamp: 'mtf 8/6/2008 00:57'!
ultimateSourceCodeAt: selector ifAbsent: aBlock
"Return the source code at selector, deferring to superclass if necessary"

	^ self sourceCodeAt: selector ifAbsent:
		[self superclassOrEditor ultimateSourceCodeAt: selector ifAbsent: aBlock]! !

!ClassDescriptionEditor methodsFor: 'validating' stamp: 'mtf 10/22/2008 21:54'!
validate
	self
		validateFormat;
		validateInstVarNames.
	super validate! !

!ClassDescriptionEditor methodsFor: 'validating' stamp: 'mtf 7/25/2008 22:56'!
validateFormat
	self edClassFormat
		validate;
		validateAgainstSuper: self edSuperFormat;
		validateAgainstOld: self edOldFormat! !

!ClassDescriptionEditor methodsFor: 'validating' stamp: 'mtf 11/26/2008 15:01'!
validateInstVarNames
	self edInstVarNames do: 
		[ :ea | 
		(ReservedNames includes: ea) ifTrue: [ IllegalVariableName signal ].
		(self superclassOrEditor edAllInstVarNames includes: ea) ifTrue: [ IllegalVariableName signal ].
		self subclassesOrEditors do: [ :class | (class edInstVarNames includes: ea) ifTrue: [ IllegalVariableName signal ] ] ]! !

!ClassDescriptionEditor methodsFor: 'reflecting' stamp: 'mtf 8/5/2008 23:57'!
withAllSuperclasses
	"Answer an OrderedCollection of the receiver and the receiver's 
	superclasses. The first element is the receiver, 
	followed by its superclass; the last element is Object."

	| temp |
	temp := self allSuperclasses.
	temp addFirst: self.
	^ temp! !

!ClassDescriptionEditor methodsFor: 'reflecting' stamp: 'mtf 10/13/2008 21:43'!
withAllSuperclassesDo: aBlock 
"Evaluate aBlock for each of my receiver superclasses."

	aBlock value: self.
	self allSuperclassesDo: aBlock! !

ClassDescriptionEditor subclass: #ClassEditor
	instanceVariableNames: 'name classVarNames sharedPools category'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Squeak'!

!ClassEditor class methodsFor: 'instance creation' stamp: 'mtf 7/11/2008 00:07'!
canEdit: anObject for: anEditor
	^ anObject isBehavior! !

!ClassEditor class methodsFor: 'instance creation' stamp: 'mtf 10/10/2008 22:21'!
classMetaclassEditor
"Answer the class of editors for my instances' cooresponding metaclass editor"

	^ MetaclassEditor! !

!ClassEditor class methodsFor: 'instance creation' stamp: 'mtf 10/7/2008 22:31'!
classRootEditor
"Answer the class of editors for the root of my heiarchy"

	^ RootClassEditor! !

!ClassEditor class methodsFor: 'instance creation' stamp: 'mtf 7/29/2008 00:21'!
new
	^ self classMetaclassEditor new new! !

!ClassEditor class methodsFor: 'util' stamp: 'gk 9/3/2007 23:41'!
rearrangeVarNames: associations oldNames: oldVars
	"Rearrange vars. The associations are name -> pos and cover the subset
	of vars that should be moved. The old ones just fill in the holes."

	| newVars |
	newVars := Array new: oldVars size.
	associations do: [:assoc |
		newVars at: assoc value put: assoc key].
	oldVars do: [:oldVar |
		(newVars includes: oldVar) ifFalse: [
			newVars at: (newVars indexOf: nil) put: oldVar]].
	^newVars! !

!ClassEditor methodsFor: 'editing' stamp: 'cwp 10/24/2005 08:27'!
addClassVarName: aSymbol
	classVarNames ifNil: [classVarNames := subject classVarNames].
	classVarNames := classVarNames copyWith: aSymbol asSymbol! !

!ClassEditor methodsFor: 'editing' stamp: 'cwp 10/23/2005 16:03'!
addSharedPool: aSharedPool 
	sharedPools ifNil: [sharedPools := subject sharedPools].
	sharedPools := sharedPools copyWith: aSharedPool! !

!ClassEditor methodsFor: 'reflecting' stamp: 'mtf 10/29/2008 13:58'!
addSubclass: aClassEditor
	(self decorateWith: SubclassListDecorator) addSubclass: aClassEditor! !

!ClassEditor methodsFor: 'reflecting' stamp: 'mtf 6/26/2008 11:25'!
binding
	^ system bindingOf: self name! !

!ClassEditor methodsFor: 'reflecting' stamp: 'mtf 10/13/2008 20:40'!
bindingOf: varName
	"Answer the binding of some variable resolved in the scope of the receiver"
	| aSymbol |
	aSymbol := varName asSymbol.

	"First look in classVar dictionary."
	(self classVarNames includes: aSymbol)
		ifTrue: [^ subject ifNotNil: [subject classPool bindingOf: aSymbol]].

	"Next look in shared pools."
	self sharedPools do: [:pool |
		(pool bindingOf: aSymbol) ifNotNilDo: [:binding | ^binding]].

	"Next look in declared environment."
	(self environment bindingOf: aSymbol) ifNotNilDo: [:binding | ^binding].

	"Finally look higher up the superclass chain and fail at the end."
	^ self superclassOrEditor bindingOf: aSymbol! !

!ClassEditor methodsFor: 'reflecting' stamp: 'mtf 2/1/2008 21:12'!
category
	"If this isn't cached, SystemEditorBrowser is really slow. And it doesn't affect the correctness of the editor, so do it by default"
	^ category ifNil: [category := subject category]! !

!ClassEditor methodsFor: 'reflecting' stamp: 'cwp 12/3/2005 09:41'!
category: aSymbol
	category := aSymbol! !

!ClassEditor methodsFor: 'editing' stamp: 'cwp 12/11/2005 22:36'!
classComment: aTextOrString stamp: aString
	self organization classComment: aTextOrString stamp: aString! !

!ClassEditor methodsFor: 'reflecting' stamp: 'cwp 11/4/2005 08:23'!
classPool
	| pool |
	pool := subject ifNil: [Dictionary new] ifNotNil: [subject classPool copy].
	classVarNames ifNil: [^ pool].
	pool keys do: [:ea | (classVarNames includes: ea) ifFalse: [pool removeKey: ea]].
	classVarNames do: [:ea | (pool includesKey: ea) ifFalse: [pool at: ea put: nil]].
	^ pool! !

!ClassEditor methodsFor: 'reflecting' stamp: 'cwp 10/23/2005 15:41'!
classVarNames
	^ classVarNames ifNil: [subject classVarNames]! !

!ClassEditor methodsFor: 'reflecting' stamp: 'cwp 12/2/2005 22:58'!
classVariablesString
	^ String streamContents: 
		[:stream | 
		self classVarNames asSortedCollection
			do: [:ea | stream nextPutAll: ea]
			separatedBy: [stream nextPut: $ ]]! !

!ClassEditor methodsFor: 'reflecting' stamp: 'mtf 10/13/2008 17:16'!
comment
	| comment |
	comment := self organization classComment.
	^ comment isEmpty
		ifFalse: [comment]
		ifTrue: [self classCommentBlank]! !

!ClassEditor methodsFor: 'reflecting' stamp: 'mtf 2/1/2008 22:44'!
definitionST80
	"Answer a String that defines the receiver."

	| aStream |
	aStream := WriteStream on: (String new: 300).
	self superclass == nil
		ifTrue: [aStream nextPutAll: 'ProtoObject']
		ifFalse: [aStream nextPutAll: self  superclass name].
	aStream nextPutAll: self kindOfSubclass;
			store: self name.
	aStream cr; tab; nextPutAll: 'instanceVariableNames: ';
			store: self instanceVariablesString.
	aStream cr; tab; nextPutAll: 'classVariableNames: ';
			store: self classVariablesString.
	aStream cr; tab; nextPutAll: 'poolDictionaries: ';
			store: self sharedPoolsString.
	aStream cr; tab; nextPutAll: 'category: ';
			store: (SystemOrganization categoryOfElement: self name) asString.

	self superclass ifNil: [ 
		aStream nextPutAll: '.'; cr.
		aStream nextPutAll: self name.
		aStream space; nextPutAll: 'superclass: nil'. ].

	^ aStream contents! !

!ClassEditor methodsFor: 'building' stamp: 'mtf 11/26/2008 14:35'!
edBuild
	| meta |
	meta := self class edBuild.
	product := subject ifNil: [meta new]
				ifNotNil: [meta adoptInstance: subject from: subject class].
	product
		superclass: self edSuperclass;
		setFormat: self format;
		setName: self name;
		setInstVarNames: self edInstVarNames;
		classPoolFrom: self;
		instVarNamed: #sharedPools put: self sharedPools;
		organization: self organization edBuild.

		"category and environment are not set here.
		category will be lazily computed sometime after 
			SystemEditor >> edRecatogorize runs. (see Class >> category)
		environment can be left nil, since it has a default value"

	"Create a temporary MethodDictionary to catch code written by
	SyntaxError dialogs. MethodDictionaryEditor will overwrite this.
	See MethodEditor>>compileFor:"
	product methodDictionary: MethodDictionary new.
	self decoratorsDo: [:ea | ea edBuildInto: product].

	"Class methods should be compiled before instance methods,
	since #compilerClass may be among the class methods.
	Class methods should be compiled after installing class and pool variables"
	product class methodDictionary: (self class methods buildFor: self class).
	product methodDictionary: (self methods buildFor: self).
	^product! !

!ClassEditor methodsFor: 'building' stamp: 'cwp 12/3/2005 18:01'!
edCategory
	^ category! !

!ClassEditor methodsFor: 'building' stamp: 'mtf 10/30/2008 19:51'!
edDependentSubclassesDo: aBlock
"Make sure my subclasses get recompiled if my bindings have changed"
	self edRequiresSubclassRebuild ifFalse: [^ #()].
	self subclassesDo: [:ea |
		ea edInvalidateSuperclass.
		aBlock value: ea]! !

!ClassEditor methodsFor: 'building' stamp: 'mtf 10/30/2008 20:04'!
edDependentSuperclassesDo: aBlock
"Fix up subclass references to myself in my old and new superclasses"

	| newSuperclass oldSuperclass newSupereditor oldSupereditor |
	newSuperclass := self superclassOrEditor.
	newSuperclass edIsEditor ifTrue: [newSuperclass := newSuperclass  subject].
	oldSuperclass := self subject ifNotNil: [self subject superclass].
	oldSuperclass == newSuperclass ifTrue: [^ self].

	newSupereditor := self environment edEditorFor: newSuperclass.
	newSupereditor addSubclass: self.
	aBlock value: newSupereditor.
	oldSuperclass ifNil: [^ self].

	oldSupereditor := self environment edEditorFor: oldSuperclass.
	oldSupereditor removeSubclass: self.
	aBlock value: oldSupereditor.! !

!ClassEditor methodsFor: 'building' stamp: 'mtf 10/30/2008 19:55'!
edDependentsDo: aBlock
	super edDependentsDo: aBlock.
	self edDependentSubclassesDo: aBlock.
	self edDependentSuperclassesDo: aBlock.! !

!ClassEditor methodsFor: 'building' stamp: 'mtf 7/22/2008 12:09'!
edName: aSymbol superclassEditor: anEditor type: typeOfClass instVarString: instString classVarString: classString poolImports: poolString category: catString 
	| scanner |
	scanner := Scanner new.
	name := aSymbol.
	self superclass: anEditor.
	type := typeOfClass.
	category := catString.
	instVarNames := scanner scanFieldNames: instString.
	classVarNames := scanner scanFieldNames: classString.
	sharedPools := (scanner scanFieldNames: poolString) 
				collect: [:ea | system edResolve: ea asSymbol]! !

!ClassEditor methodsFor: 'building' stamp: 'mtf 9/4/2007 22:21'!
edNameHasChanged
	name ifNil: [^ false].
	^ name ~~ self subject name! !

!ClassEditor methodsFor: 'building' stamp: 'mtf 10/29/2008 13:59'!
edPostLoad
	self decoratorsDo: [:ea | ea edPostLoad]! !

!ClassEditor methodsFor: 'building' stamp: 'cwp 3/21/2006 15:36'!
edPrepareExport: exporter 
	^exporter addClass: self product! !

!ClassEditor methodsFor: 'building' stamp: 'cwp 4/15/2006 20:19'!
edPrepareInstanceMigration: txn
	txn addMigrator: (InstanceMigrator from: self subject to: self product)! !

!ClassEditor methodsFor: 'building' stamp: 'mtf 10/15/2008 18:42'!
edPrepareMigration: txn
	self subject ifNil: [^ self].
	self edRequiresBuild 
		ifTrue: [self edPrepareInstanceMigration: txn]
		ifFalse: [super edPrepareMigration: txn]! !

!ClassEditor methodsFor: 'building' stamp: 'mtf 11/26/2008 18:37'!
edRequiresBuild
	self edRequiresRecompile ifTrue: [^ true].
	self edNameHasChanged ifTrue: [^ true].
	self class edRequiresBuild ifTrue: [^ true].
	self decoratorsDo: [:ea | ea edRequiresBuild ifTrue: [^ true]].
	^ false! !

!ClassEditor methodsFor: 'building' stamp: 'cwp 11/7/2005 19:54'!
edRequiresMigration
	^ subject
		ifNotNil: [self format ~= subject format]
		ifNil: [false]! !

!ClassEditor methodsFor: 'building' stamp: 'mtf 10/15/2008 19:03'!
edRequiresRecompile
	classVarNames ifNotNil: [^ true].
	sharedPools ifNotNil: [^ true].
	^ super edRequiresRecompile! !

!ClassEditor methodsFor: 'building' stamp: 'cwp 11/7/2005 20:00'!
edRequiresSubclassRebuild
	^ self class edRequiresSubclassRebuild 
		or: [self edRequiresMigration]
		or: [self edRequiresRecompile]! !

!ClassEditor methodsFor: 'editing' stamp: 'kph 5/30/2007 16:47'!
ensureClassVarName: aString 
	(self allClassVarNames includes: aString)
		ifFalse: [ self addClassVarName: aString].
		 ! !

!ClassEditor methodsFor: 'editing' stamp: 'kph 5/30/2007 16:48'!
ensureSharedPoolName: aString 
	(self allSharedPools includes: aString)
		ifFalse: [ self addSharedPoolName: aString].
		 ! !

!ClassEditor methodsFor: 'accessing' stamp: 'mtf 11/19/2008 21:46'!
ensuredCompilerClass
"Answer my  product's compilerClass, building it if necessary"

	| methodEditor method |
	super ensuredCompilerClass ifNotNil: [^ compilerClass].

	"If we get here, this package is using a compiler that does not yet exist in the system. 
	Compile and run the #compilerClass method it in the context of the system editor.
	Due to how ClassEditor implements #bindingOf:, this will answer the editor for the right compiler"
	methodEditor := self class edUltimateMethodAt: #compilerClass.
	method := methodEditor compileForClass: self using: Compiler new.
	compilerClass := (self new executeMethod: method) product.
	compilerClass ifNil: [self error: 'Unknown compiler'].
	^ compilerClass! !

!ClassEditor methodsFor: 'reflecting' stamp: 'cwp 12/11/2005 13:32'!
hasComment
	| comment |
	comment := self organization classComment.
	^ comment notNil and: [comment isEmpty not]! !

!ClassEditor methodsFor: 'debugging' stamp: 'mtf 9/4/2007 20:01'!
instVarAt: index
	"Override the implemenation in Object. That will return the inst vars of 
	ClassEditor, but we want the inst vars of the subject."

	self class isDebuggingAsEditor ifTrue: [^ super instVarAt: index].
	
	"If the insance variable has not yet been installed, return nil"
	self subject ifNil: [^ nil].
	index > self subject class instSize ifTrue: [^ nil].
	
	^ self subject instVarAt: index! !

!ClassEditor methodsFor: 'debugging' stamp: 'mtf 9/4/2007 20:01'!
instVarAt: index put: anObject
	"Override the implemenation in Object. That will set the inst vars of 
	ClassEditor, but we want the inst vars of the subject."

	self class isDebuggingAsEditor ifTrue: [^ super instVarAt: index put: anObject].
	
	"If the insance variable has not yet been installed, return nil"
	self subject ifNil: [^ self].
	index > self subject class instSize ifTrue: [^ self].
	
	^ self subject instVarAt: index put: anObject! !

!ClassEditor methodsFor: 'testing' stamp: 'mtf 9/13/2007 19:54'!
isMeta
	^ false! !

!ClassEditor methodsFor: 'accessing' stamp: 'gk 9/5/2007 11:03'!
name
	^ name ifNil: [subject name]! !

!ClassEditor methodsFor: 'reflecting' stamp: 'mtf 10/13/2008 21:09'!
new
"Create an instance of myself. This instance is only intended for compiling DoIts that will evaluate in my scope. See SystemEditor >> doItHost"

	superclass := self subject ifNil: [UndefinedObject].
	methodDict := MethodDictionary new.
	format := superclass format.
	^ self basicNew! !

!ClassEditor methodsFor: 'initialize-release' stamp: 'mtf 6/27/2008 10:05'!
on: aClass for: aSystemEditor
	^ self setSubject: aClass system: aSystemEditor! !

!ClassEditor methodsFor: 'accessing' stamp: 'mtf 10/13/2008 22:40'!
realClass
"Answer the true class, which is the superclass of my MetaclassEditor"

	^ self class instVarAt: 1! !

!ClassEditor methodsFor: 'editing' stamp: 'gk 9/3/2007 23:44'!
rearrangeClassVarNames: associations
	"The associations are name -> pos and cover the subset
	of vars that should be moved. The old just fill in the holes."

	classVarNames := ClassEditor rearrangeVarNames: associations oldNames: classVarNames! !

!ClassEditor methodsFor: 'editing' stamp: 'gk 9/5/2007 00:22'!
rearrangeSharedPoolVarNames: associations
	"The associations are name -> pos and cover the subset
	of vars that should be moved. The old just fill in the holes.
	NOTE: This method uses identityIncludes:, otherwise it fails on empty Dictionaries."

	| newPools |
	newPools := Array new: sharedPools size.
	associations do: [:assoc |
		newPools at: assoc value put: (
			system edResolve: assoc key asSymbol
				ifAbsent: [self error: 'Can not resolve pool ', assoc key])].
	sharedPools do: [:oldPool |
		(newPools identityIncludes: oldPool) ifFalse: [
			newPools at: (newPools indexOf: nil) put: oldPool]].
	sharedPools := newPools! !

!ClassEditor methodsFor: 'editing' stamp: 'cwp 10/23/2005 15:58'!
removeClassVarName: aString 
	classVarNames ifNil: [classVarNames := subject classVarNames].
	classVarNames := classVarNames copyWithout: aString! !

!ClassEditor methodsFor: 'editing' stamp: 'cwp 12/3/2005 20:24'!
removeFromSystem
	system edRemoveClassNamed: self name! !

!ClassEditor methodsFor: 'editing' stamp: 'gk 9/5/2007 00:10'!
removeSharedPool: aSharedPool
	"We use reject based on identity instead of copyWithout: here."

	sharedPools ifNil: [sharedPools := subject sharedPools].
	sharedPools := sharedPools reject: [:each | each == aSharedPool]! !

!ClassEditor methodsFor: 'reflecting' stamp: 'mtf 10/29/2008 13:58'!
removeSubclass: aClassEditor
	(self decorateWith: SubclassListDecorator) removeSubclass: aClassEditor! !

!ClassEditor methodsFor: 'editing' stamp: 'mtf 11/17/2007 23:58'!
rename: aString
	| oldName newName |
	(newName := aString asSymbol) = (oldName := self name)
		ifTrue: [^ self].
	(self system includesKey: newName)
		ifTrue: [^ self error: newName , ' already exists'].
	self setName: newName.
	self category: self category. "set explicitly to trigger reCategorize"
	self system renameClass: self from: oldName! !

!ClassEditor methodsFor: 'editing' stamp: 'cwp 11/4/2005 08:14'!
setName: aSymbol
	name := aSymbol! !

!ClassEditor methodsFor: 'initialize-release' stamp: 'mtf 10/16/2008 18:55'!
setSubject: aClass system: aSystemEditor 
	subject := aClass.
	system := aSystemEditor. 
	subject ifNil: 
			[superEditor := aSystemEditor at: #Object.
			name := #Annonymous.
			type := #normal.
			instVarNames := Array new.
			classVarNames := Array new.
			sharedPools := Array new.
			methods := MethodDictionaryEditor for: self.
			self class edInitializeForNewClass]! !

!ClassEditor methodsFor: 'reflecting' stamp: 'cwp 10/24/2005 18:57'!
sharedPools
	^ sharedPools ifNil: [subject sharedPools]! !

!ClassEditor methodsFor: 'reflecting' stamp: 'cwp 12/2/2005 23:11'!
sharedPoolsString
	^ String streamContents: [ :stream |
		self sharedPools 
			do: [ :each |
				stream nextPutAll: (system 
								keyAtIdentityValue: each 
								ifAbsent: ['private'])]
			separatedBy: [stream space]]! !

!ClassEditor methodsFor: 'editing' stamp: 'kph 5/26/2007 05:58'!
subclass: aSymbol instanceVariableNames: instString classVariableNames: classString poolDictionaries: poolString category: categoryString 
	^ ((system 
		at: aSymbol
		ifAbsent: [ ClassEditor on: nil for: system])
			edName: aSymbol
			superclassEditor: self
			type: #normal
			instVarString: instString
			classVarString: classString
			poolImports: poolString
			category: categoryString) edRegisterEditor! !

!ClassEditor methodsFor: 'reflecting' stamp: 'cwp 11/23/2005 21:51'!
subclasses
	^ system edSubclassesOf: self! !

!ClassEditor methodsFor: 'reflecting' stamp: 'mtf 10/13/2008 23:03'!
subclassesDo: aBlock
	^ system edSubclassesOf: self do: aBlock! !

!ClassEditor methodsFor: 'reflecting' stamp: 'mtf 9/18/2008 12:08'!
subclassesOrEditors
"Answer all of my subclasses, as either classes or editors, whichever is easier"

	^ system edSubclassesOrEditorsOf: self! !

!ClassEditor methodsFor: 'reflecting' stamp: 'mtf 10/13/2008 23:04'!
subclassesOrEditorsDo: aBlock
"Evaluates aBlock for all of my subclasses, as either classes or editors, whichever is easier"

	system edSubclassesOrEditorsOf: self do: aBlock! !

!ClassEditor methodsFor: 'editing' stamp: 'mtf 9/20/2008 07:20'!
superclass: aClassOrEditor 
	super superclass: aClassOrEditor.
	self class superclass: aClassOrEditor class! !

!ClassEditor methodsFor: 'reflecting' stamp: 'mtf 9/13/2007 19:53'!
theMetaClass
	^ self class! !

!ClassEditor methodsFor: 'reflecting' stamp: 'mtf 9/13/2007 19:53'!
theNonMetaClass
	^ self! !

ClassEditor subclass: #RootClassEditor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Squeak'!

!RootClassEditor commentStamp: 'mtf 7/29/2008 08:37' prior: 0!
My instances represent ClassEditors on the concept of "ProtoObject's superclass", the primitive object from which every class, without exception, inherits. I exist so that queries that involve the superclass do not have to be special-cased in case the superclass really is nil.!

!RootClassEditor class methodsFor: 'instance creation' stamp: 'mtf 7/29/2008 08:32'!
canEdit: anObject for: anEditor
	^ false! !

!RootClassEditor class methodsFor: 'instance creation' stamp: 'mtf 10/13/2008 22:28'!
classMetaclassEditor
	^ RootMetaclassEditor! !

!RootClassEditor methodsFor: 'reflecting' stamp: 'mtf 10/13/2008 18:37'!
bindingOf: varName
	^ nil! !

!RootClassEditor methodsFor: 'accessing' stamp: 'mtf 11/26/2008 15:05'!
edAllInstVarNames
	^ #()! !

!RootClassEditor methodsFor: 'accessing' stamp: 'mtf 11/26/2008 15:06'!
edInstSize
	^ 0! !

!RootClassEditor methodsFor: 'initialize-release' stamp: 'mtf 7/28/2008 14:06'!
edRegisterEditor
"Do nothing"! !

!RootClassEditor methodsFor: 'building' stamp: 'mtf 7/28/2008 13:55'!
edRequiresBuild
	^ false! !

!RootClassEditor methodsFor: 'building' stamp: 'mtf 7/28/2008 14:02'!
edRequiresRecompile
	^ false! !

!RootClassEditor methodsFor: 'accessing' stamp: 'mtf 11/18/2008 11:45'!
edUltimateMethodAt: aSelector ifAbsent: aBlock
	^ aBlock value! !

!RootClassEditor methodsFor: 'building' stamp: 'mtf 7/28/2008 14:03'!
product
	^ nil! !

!RootClassEditor methodsFor: 'accessing' stamp: 'mtf 7/28/2008 14:03'!
subject
	^ nil! !

!RootClassEditor methodsFor: 'reflecting' stamp: 'mtf 7/28/2008 14:03'!
typeOfClass
	^ #normal! !

!RootClassEditor methodsFor: 'reflecting' stamp: 'mtf 8/6/2008 00:54'!
ultimateSourceCodeAt: selector ifAbsent: aBlock
"Return the source code at selector, deferring to superclass if necessary"

	^ aBlock value! !

!RootClassEditor methodsFor: 'reflecting' stamp: 'mtf 10/13/2008 21:44'!
withAllSuperclasses
	^ OrderedCollection new! !

!RootClassEditor methodsFor: 'reflecting' stamp: 'mtf 10/13/2008 21:46'!
withAllSuperclassesDo: aBlock
	^ self! !

ClassDescriptionEditor subclass: #MetaclassEditor
	instanceVariableNames: 'editor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Squeak'!

!MetaclassEditor class methodsFor: 'instance creation' stamp: 'mtf 8/13/2008 00:34'!
classClassEditor
"Answers the class of my instance-side editor"

	^ ClassEditor! !

!MetaclassEditor class methodsFor: 'instance creation' stamp: 'mtf 10/7/2008 22:31'!
classRootEditor
"Answer the class of editors for the root of my heiarchy"

	^ RootMetaclassEditor! !

!MetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 8/13/2008 00:34'!
acceptsLoggingOfCompilation

	^ self instanceSide acceptsLoggingOfCompilation! !

!MetaclassEditor methodsFor: 'debugging' stamp: 'mtf 10/15/2008 13:46'!
addSelectorSilently: selector withMethod: compiledMethod
"For running do-its against my ClassEditor"

	^ superclass addSelectorSilently: selector withMethod: compiledMethod! !

!MetaclassEditor methodsFor: 'debugging' stamp: 'mtf 10/13/2008 22:23'!
allInstVarNames
	 "specialized in order to enable debugger to show as self"
	^ self isDebuggingAsEditor
		ifTrue: [superclass allInstVarNames]
		ifFalse: [super allInstVarNames]! !

!MetaclassEditor methodsFor: 'debugging' stamp: 'mtf 10/15/2008 13:47'!
basicRemoveSelector: selector
"For running do-its against my ClassEditor"

	^ superclass basicRemoveSelector: selector! !

!MetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 10/13/2008 18:39'!
bindingOf: varName
	^ editor bindingOf: varName! !

!MetaclassEditor methodsFor: 'debugging' stamp: 'mtf 10/15/2008 13:40'!
canUnderstand: selector

	^ superclass canUnderstand: selector! !

!MetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 8/13/2008 00:34'!
classPool
	^ editor classPool! !

!MetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 8/13/2008 00:34'!
classSide
	^self theMetaClass! !

!MetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 8/13/2008 00:34'!
comment
	^ self instanceSide comment! !

!MetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 8/13/2008 00:34'!
definitionST80
	^ String streamContents: 
		[:strm |
		strm print: self;
			crtab;
			nextPutAll: 'instanceVariableNames: ';
			store: self instanceVariablesString]! !

!MetaclassEditor methodsFor: 'building' stamp: 'mtf 11/26/2008 15:49'!
edBuild
	| class |
	class := self subject ifNil: [Metaclass] ifNotNil: [self subject class].
	product := class basicNew.
	"Create a temporary MethodDictionary to catch code written by SyntaxError dialogs. MethodDictionaryEditor will overwrite this. See MethodEditor>>compileFor:"
	product
		superclass: self edSuperclass
		methodDictionary: MethodDictionary new
		format: self format;		
		organization: self organization edBuild.
	product setInstVarNames: self edInstVarNames.
	self decoratorsDo: [:ea | ea edBuildInto: product].
	^ product! !

!MetaclassEditor methodsFor: 'building' stamp: 'mtf 8/13/2008 00:34'!
edInitializeForNewClass
	superEditor := self system edEditorFor: Object class.
	instVarNames := Array new.
	methods := MethodDictionaryEditor for: self! !

!MetaclassEditor methodsFor: 'building' stamp: 'mtf 8/13/2008 00:34'!
edRequiresBuild
	^ (superEditor ~~ nil) | (instVarNames ~~ nil) | (methods ~~ nil)! !

!MetaclassEditor methodsFor: 'building' stamp: 'mtf 10/13/2008 15:55'!
edRequiresSubclassRebuild
	^ self subject
		ifNotNil: [self subject format ~= self format]
		ifNil: [false]! !

!MetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 8/13/2008 00:34'!
hasComment
	^ self instanceSide hasComment! !

!MetaclassEditor methodsFor: 'debugging' stamp: 'mtf 10/13/2008 21:23'!
inheritsFrom: aClass
"This allows aClassEditor isKindOf: Object to work properly"

	^ superclass inheritsFrom: aClass! !

!MetaclassEditor methodsFor: 'initialization' stamp: 'mtf 10/13/2008 22:19'!
initialize
"initialize myself so that I can create instances"

	superclass := self class classClassEditor.
	methodDict := MethodDictionary new.
	format := superclass format.! !

!MetaclassEditor methodsFor: 'debugging' stamp: 'mtf 10/13/2008 22:24'!
instSize
	"Override the implemenation in Behavior, so that when inspecting the
	ClassEditor, it will look like a regular Class"

	^ self isDebuggingAsEditor
		ifTrue: [superclass instSize]
		ifFalse: [super instSize]! !

!MetaclassEditor methodsFor: 'debugging' stamp: 'mtf 10/13/2008 22:24'!
instVarNames
	"Override the implemenation in Behavior. That will return the inst vars of 
	ClassEditor, but we want the inst vars of the subject's class."

	^ self isDebuggingAsEditor
		ifTrue: [superclass instVarNames]
		ifFalse: [super instVarNames]! !

!MetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 8/13/2008 00:34'!
instanceSide
	^ self theNonMetaClass! !

!MetaclassEditor methodsFor: 'debugging' stamp: 'mtf 9/18/2008 10:47'!
isDebuggingAsEditor
	"To be transparent enough to fool most existing software,
	MetaclassEditors must answer their subject's instance variables when
	sent the message allInstVarNames. However, this message is also used by
	the debugger, inspector, and explorer, when inspecting ClassEditors, and
	an inspector should see the real instance variables of ClassEditor rather
	than those of the MetaclassEditor's subject. So there is a conflict of
	interest already.
	
	But it gets worse. If you do open an inspector on an object that is
	reporting wrong information about it's instance variables, Bad things
	happen. Specifically, the inspector calls instVarAt: on an instance
	variable that does not exist, leading to an error every time the display
	is updated. Less severe, instVarAt: will answer nonsensical. 
	
	So I took the instance variable proxying behavior all the way so that it
	successfully fools the debugger, inspector, and explorer into seeing
	proxied instance variables (see the senders af this message if you want
	to know how). But there is still the original problem that you, the
	programmer want to see the real state of ClassEditor. That is where this
	method comes in.
	
	If this method returns false, everybody sees the proxied instance
	variables. This is the correct behavior. If this method returns false,
	everyone sees the true instance variables of ClassEditor. This is
	incorrect behavior, and breaks the Unit tests, but it is invaluable if
	you are debugging ClassEditor"

	^ self environment debug! !

!MetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 8/13/2008 00:34'!
isMeta
	^ true! !

!MetaclassEditor methodsFor: 'debugging' stamp: 'mtf 8/13/2008 00:34'!
isUniClass
	"The debugger sometimes sends this message"
	^ false! !

!MetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 8/13/2008 00:34'!
name
	^ editor name, ' class'! !

!MetaclassEditor methodsFor: 'initialization' stamp: 'mtf 10/13/2008 21:01'!
new
"Answers a new ClassEditor with myself as its class"
	^ editor := self basicNew! !

!MetaclassEditor methodsFor: 'building' stamp: 'mtf 10/15/2008 18:37'!
product
	^ product ifNil: [product := editor product class]! !

!MetaclassEditor methodsFor: 'debugging' stamp: 'mtf 10/15/2008 13:34'!
selectorAtMethod: method setClass: classResultBlock
"For getting the class of a method prior to 3.9"

	^ superclass selectorAtMethod: method setClass: classResultBlock! !

!MetaclassEditor methodsFor: 'debugging' stamp: 'mtf 10/15/2008 13:43'!
sharedPools

	^ superclass sharedPools! !

!MetaclassEditor methodsFor: 'accessing' stamp: 'mtf 10/15/2008 13:51'!
subject
	subject ifNotNil: [^ subject].
	editor subject ifNil: [^ nil].
	^ subject := editor subject class! !

!MetaclassEditor methodsFor: 'accessing' stamp: 'mtf 10/13/2008 20:28'!
system
	^ system ifNil: [system := editor system]! !

!MetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 8/13/2008 00:34'!
theMetaClass
	^ self! !

!MetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 8/13/2008 00:34'!
theNonMetaClass
	^ editor! !

!MetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 8/13/2008 00:34'!
typeOfClass
	^ #normal! !

MetaclassEditor subclass: #RootMetaclassEditor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Squeak'!

!RootMetaclassEditor class methodsFor: 'instance creation' stamp: 'mtf 8/13/2008 00:35'!
canEdit: anObject for: anEditor
	^ false! !

!RootMetaclassEditor class methodsFor: 'instance creation' stamp: 'mtf 8/13/2008 00:35'!
classClassEditor
	^ RootClassEditor! !

!RootMetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 10/13/2008 18:37'!
bindingOf: varName
	^ nil! !

!RootMetaclassEditor methodsFor: 'accessing' stamp: 'mtf 11/26/2008 15:06'!
edAllInstVarNames
	^ #()! !

!RootMetaclassEditor methodsFor: 'accessing' stamp: 'mtf 11/26/2008 15:08'!
edInstSize
	^ 0! !

!RootMetaclassEditor methodsFor: 'initialize-release' stamp: 'mtf 8/13/2008 00:35'!
edRegisterEditor
"Do nothing"! !

!RootMetaclassEditor methodsFor: 'building' stamp: 'mtf 8/13/2008 00:35'!
edRequiresBuild
	^ false! !

!RootMetaclassEditor methodsFor: 'building' stamp: 'mtf 8/13/2008 00:35'!
edRequiresRecompile
	^ false! !

!RootMetaclassEditor methodsFor: 'accessing' stamp: 'mtf 11/18/2008 11:46'!
edUltimateMethodAt: aSelector ifAbsent: aBlock
	^ aBlock value! !

!RootMetaclassEditor methodsFor: 'building' stamp: 'mtf 8/13/2008 00:35'!
product
	^ nil! !

!RootMetaclassEditor methodsFor: 'accessing' stamp: 'mtf 8/13/2008 00:35'!
subject
	^ nil! !

!RootMetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 8/13/2008 00:35'!
typeOfClass
	^ #normal! !

!RootMetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 8/13/2008 00:35'!
ultimateSourceCodeAt: selector ifAbsent: aBlock
"Return the source code at selector, deferring to superclass if necessary"

	^ aBlock value! !

!RootMetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 10/13/2008 21:45'!
withAllSuperclasses
	^ OrderedCollection new! !

!RootMetaclassEditor methodsFor: 'reflecting' stamp: 'mtf 10/13/2008 21:45'!
withAllSuperclassesDo: aBlock
	^ self! !

!PureBehaviorEditor class methodsFor: 'accessing' stamp: 'mtf 10/13/2008 20:20'!
reservedNames
	^ ReservedNames! !

!PureBehaviorEditor methodsFor: 'reflecting' stamp: 'mtf 10/13/2008 15:42'!
acceptsLoggingOfCompilation

	self subject ifNil: [ ^ true ].
	^ self subject acceptsLoggingOfCompilation! !

!PureBehaviorEditor methodsFor: 'reflecting' stamp: 'mtf 10/13/2008 21:53'!
binding
	^ nil -> self! !

!PureBehaviorEditor methodsFor: 'reflecting' stamp: 'mtf 10/13/2008 18:40'!
bindingOf: varName
	^ self environment bindingOf: varName! !

!PureBehaviorEditor methodsFor: 'reflecting' stamp: 'mtf 10/13/2008 17:17'!
classCommentBlank
	^ 'Main comment stating the purpose of this class and relevant relationship to other classes.

Possible useful expressions for doIt or printIt.

Structure:
 instVar1		type -- comment about the purpose of instVar1
 instVar2		type -- comment about the purpose of instVar2

Any further useful comments about the general approach of this implementation.'! !

!PureBehaviorEditor methodsFor: 'editing' stamp: 'mtf 10/13/2008 15:36'!
compile: source classified: cat
	^ self 
		compile: source
		classified: cat
		withStamp: Utilities changeStamp
		notifying: nil
		logSource: self acceptsLoggingOfCompilation ! !

!PureBehaviorEditor methodsFor: 'editing' stamp: 'mtf 10/13/2008 15:37'!
compile: source classified: cat notifying: anObject
	^ self 
		compile: source
		classified: cat
		withStamp: Utilities changeStamp
		notifying: anObject
		logSource: self acceptsLoggingOfCompilation ! !

!PureBehaviorEditor methodsFor: 'editing' stamp: 'mtf 10/13/2008 15:37'!
compile: source classified: cat withStamp: aString notifying: anObject
	^ self 
		compile: source
		classified: cat
		withStamp: aString
		notifying: anObject
		logSource: self acceptsLoggingOfCompilation ! !

!PureBehaviorEditor methodsFor: 'editing' stamp: 'mtf 10/13/2008 15:37'!
compile: source classified: cat withStamp: aString notifying: anObject logSource: aBoolean
	| cm |
	cm := MethodEditor 
			source: source
			classified: cat
			stamp: aString
			notifying: anObject
			logging: aBoolean.
	self methods add: cm.
	^ cm selector! !

!PureBehaviorEditor methodsFor: 'editing' stamp: 'mtf 10/13/2008 15:37'!
compiledMethodAt: aSelector

	^ self compiledMethodAt: aSelector ifAbsent: [ self error: 'key not found' ]! !

!PureBehaviorEditor methodsFor: 'editing' stamp: 'mtf 10/13/2008 15:37'!
compiledMethodAt: aSelector ifAbsent: aBlock
	^ self edMethodAt: aSelector ifAbsent: aBlock! !

!PureBehaviorEditor methodsFor: 'reflecting' stamp: 'mtf 10/13/2008 16:23'!
compilerClass
	^ self subject 
		ifNil: [Compiler]
		ifNotNil: [self subject compilerClass]! !

!PureBehaviorEditor methodsFor: 'reflecting' stamp: 'mtf 10/13/2008 21:31'!
decompilerClass
	^ self subject 
		ifNil: [Decompiler]
		ifNotNil: [self subject decompilerClass]! !

!PureBehaviorEditor methodsFor: 'accessing decorators' stamp: 'mtf 10/16/2008 18:58'!
decorateWith: aClass
	decorators ifNil: [decorators := IdentityDictionary new].
	^ decorators at: aClass ifAbsentPut: [aClass for: self]! !

!PureBehaviorEditor methodsFor: 'accessing decorators' stamp: 'mtf 10/13/2008 15:40'!
decoratorsDo: aBlock
	decorators ifNotNil: [decorators do: aBlock]! !

!PureBehaviorEditor methodsFor: 'building' stamp: 'mtf 10/30/2008 13:28'!
edDependentsDo: aBlock
"Find any editors that should be modified because of me, inform them if necessary, then send them to aBlock"
	self decoratorsDo: [:ea | ea edDependentsDo: aBlock]! !

!PureBehaviorEditor methodsFor: 'building' stamp: 'mtf 10/28/2008 23:27'!
edInvalidateOrganization
"Mark my organization as requiring a rebuild, by setting it non-nil"
	self organization! !

!PureBehaviorEditor methodsFor: 'testing' stamp: 'mtf 10/15/2008 18:22'!
edIsBehavior
"Answers true if I am an editor for a Behavior"
	^ false! !

!PureBehaviorEditor methodsFor: 'editing' stamp: 'mtf 10/13/2008 16:27'!
edMethodAt: aSelector
	^ self methods at: aSelector! !

!PureBehaviorEditor methodsFor: 'editing' stamp: 'mtf 10/13/2008 16:27'!
edMethodAt: aSelector ifAbsent: aBlock
	^ self methods at: aSelector ifAbsent: aBlock! !

!PureBehaviorEditor methodsFor: 'accessing' stamp: 'mtf 10/13/2008 15:41'!
edMethodsDo: aBlock
	methods ifNotNil: [methods do: aBlock]! !

!PureBehaviorEditor methodsFor: 'building' stamp: 'mtf 10/28/2008 22:40'!
edPrepareMigration: txn
	self subject ifNil: [^ self].
	self decoratorsDo: [:ea | ea edPrepareMigration: txn].
	methods ifNotNil: [methods edPrepareMigration: txn for: self].
	organization ifNotNil: [organization edPrepareMigration: txn].
! !

!PureBehaviorEditor methodsFor: 'building' stamp: 'mtf 10/15/2008 18:46'!
edRequiresBuild
"Answer true if this editor needs building; false otherwise"
	^ self subject isNil! !

!PureBehaviorEditor methodsFor: 'editing' stamp: 'mtf 11/18/2008 12:11'!
edUltimateMethodAt: aSelector
	^ self edUltimateMethodAt: aSelector ifAbsent: [self error: 'Method not found']! !

!PureBehaviorEditor methodsFor: 'editing' stamp: 'mtf 11/18/2008 11:42'!
edUltimateMethodAt: aSelector ifAbsent: aBlock
	^ self edMethodAt: aSelector ifAbsent: aBlock! !

!PureBehaviorEditor methodsFor: 'accessing' stamp: 'mtf 11/19/2008 21:41'!
ensuredCompilerClass
"Answer the compiler for my methods, building it if necessary"

	compilerClass ifNotNil: [^ compilerClass].

	"Check to see if I have a newer version of the compiler"
	self product compilerClass ifNotNil: [
		compilerClass := self system classOrEditorFor: self product compilerClass.
		compilerClass edIsEditor ifTrue: [compilerClass := compilerClass product]].
	^ compilerClass! !

!PureBehaviorEditor methodsFor: 'accessing' stamp: 'mtf 10/13/2008 15:46'!
environment
	^ self system! !

!PureBehaviorEditor methodsFor: 'reflecting' stamp: 'mtf 10/13/2008 21:51'!
evaluatorClass
	^ self subject 
		ifNil: [Compiler]
		ifNotNil: [self subject evaluatorClass]! !

!PureBehaviorEditor methodsFor: 'accessing properties' stamp: 'mtf 10/13/2008 15:37'!
hasProperty: aSymbol
	self propertyAt: aSymbol ifAbsent: [^ false].
	^ true! !

!PureBehaviorEditor methodsFor: 'reflecting' stamp: 'mtf 10/13/2008 16:24'!
includesSelector: aSelector
	^ (methods ifNil: [self subject] ifNotNil: [methods])
		includesSelector: aSelector! !

!PureBehaviorEditor methodsFor: 'reflecting' stamp: 'mtf 10/13/2008 23:20'!
literalScannedAs: literal notifying: anEncoder 
	literal isVariableBinding ifTrue: [self halt].
	^ literal! !

!PureBehaviorEditor methodsFor: 'accessing' stamp: 'mtf 10/13/2008 15:40'!
methods
	^ methods ifNil: [methods := MethodDictionaryEditor for: self]! !

!PureBehaviorEditor methodsFor: 'accessing' stamp: 'mtf 10/13/2008 15:40'!
organization
	^ organization ifNil: [organization := OrganizationEditor for: self]! !

!PureBehaviorEditor methodsFor: 'reflecting' stamp: 'mtf 10/13/2008 21:31'!
parserClass
	^ self subject 
		ifNil: [Parser]
		ifNotNil: [self subject parserClass]! !

!PureBehaviorEditor methodsFor: 'printing' stamp: 'mtf 10/13/2008 22:43'!
printOn: aStream

	| title |
	title := self realClass name.
	aStream
		nextPutAll: (title first isVowel ifTrue: ['an '] ifFalse: ['a ']);
		nextPutAll: title;
		nextPutAll: ' on: ';
		print: self subject ! !

!PureBehaviorEditor methodsFor: 'building' stamp: 'mtf 10/15/2008 18:45'!
product
	^ product ifNil: [product := self edRequiresBuild ifTrue: [self edBuild] ifFalse: [subject]]! !

!PureBehaviorEditor methodsFor: 'accessing properties' stamp: 'mtf 10/13/2008 15:38'!
propertyAt: key

	^self propertyAt: key ifAbsent: [nil]! !

!PureBehaviorEditor methodsFor: 'accessing properties' stamp: 'mtf 10/13/2008 15:38'!
propertyAt: key ifAbsent: aBlock

	properties ifNil: [^ aBlock value].
	^ properties at: key ifAbsent: aBlock! !

!PureBehaviorEditor methodsFor: 'accessing properties' stamp: 'mtf 10/13/2008 15:38'!
propertyAt: key ifAbsentPut: aBlock

	^ self propertyAt: key ifAbsent: [
		self propertyAt: key put: aBlock value]! !

!PureBehaviorEditor methodsFor: 'accessing properties' stamp: 'mtf 10/15/2008 19:21'!
propertyAt: key ifPresent: aBlock

	self propertyAt: key ifAbsent: [^ nil].
	^ aBlock value! !

!PureBehaviorEditor methodsFor: 'accessing properties' stamp: 'mtf 10/15/2008 19:22'!
propertyAt: key ifPresentDo: aBlock

	| value |
	value := self propertyAt: key ifAbsent: [^ nil].
	^ aBlock value: value! !

!PureBehaviorEditor methodsFor: 'accessing properties' stamp: 'mtf 10/13/2008 15:38'!
propertyAt: aSymbol put: value

	properties ifNil: [properties := Dictionary new].
	^ properties at: aSymbol put: value! !

!PureBehaviorEditor methodsFor: 'accessing' stamp: 'mtf 10/13/2008 22:39'!
realClass
"Since the metaobject protocol gets pretty messed up in some of my subclasses, answer what class this really is an instance of"

	^ self class! !

!PureBehaviorEditor methodsFor: 'editing' stamp: 'mtf 10/13/2008 16:28'!
removeSelector: aSymbol 
	self methods remove: aSymbol.! !

!PureBehaviorEditor methodsFor: 'reflecting' stamp: 'mtf 10/13/2008 16:24'!
selectors
	^ self organization allMethodSelectors! !

!PureBehaviorEditor methodsFor: 'reflecting' stamp: 'mtf 10/13/2008 16:24'!
sourceCodeAt: aSelector
	^ self sourceCodeAt: aSelector ifAbsent: [nil]! !

!PureBehaviorEditor methodsFor: 'reflecting' stamp: 'mtf 10/13/2008 16:24'!
sourceCodeAt: aSelector ifAbsent: aBlock
	^ (self edMethodAt: aSelector ifAbsent: aBlock) source! !

!PureBehaviorEditor methodsFor: 'reflecting' stamp: 'mtf 10/13/2008 16:25'!
sourceCodeTemplate
	^ self class sourceCodeTemplate! !

!PureBehaviorEditor methodsFor: 'accessing' stamp: 'mtf 10/13/2008 17:03'!
subject
	^ subject! !

!PureBehaviorEditor methodsFor: 'accessing' stamp: 'mtf 10/13/2008 15:48'!
system
	^ system! !

!PureBehaviorEditor methodsFor: 'reflecting' stamp: 'mtf 10/13/2008 16:27'!
ultimateSourceCodeAt: selector ifAbsent: aBlock
	"Return the source code at selector"
	
	^self sourceCodeAt: selector ifAbsent: aBlock! !

!PureBehaviorEditor methodsFor: 'validating' stamp: 'mtf 10/22/2008 21:55'!
validate
	self decoratorsDo: [:ea | ea validate]! !

!PureBehaviorEditor methodsFor: 'reflecting' stamp: 'mtf 10/13/2008 16:25'!
whichCategoryIncludesSelector: aSymbol 
	^ self organization categoryOfElement: aSymbol! !

ClassDecorator subclass: #SubclassListDecorator
	instanceVariableNames: 'parent additions removals'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Squeak'!

!SubclassListDecorator commentStamp: 'mtf 10/29/2008 13:56' prior: 0!
I remember what subclasses were added to or removed from my parent. Since the subclass list can be nil at times, I cannot migrate it, so I work in edPostLoad!

!SubclassListDecorator methodsFor: 'accessing' stamp: 'mtf 10/29/2008 13:43'!
addSubclass: aClassEditor
	additions add: aClassEditor.
	removals remove: aClassEditor ifAbsent: [].! !

!SubclassListDecorator methodsFor: 'building' stamp: 'mtf 10/29/2008 13:56'!
edPostLoad
	| product |
	product := parent product.
	removals do: [:ea | product removeSubclass: ea product].
	additions do: [:ea | product addSubclass: ea product].! !

!SubclassListDecorator methodsFor: 'initialize-release' stamp: 'mtf 10/29/2008 13:46'!
for: aClassEditor
	parent := aClassEditor.
	additions := IdentitySet new.
	removals := IdentitySet new.! !

!SubclassListDecorator methodsFor: 'accessing' stamp: 'mtf 10/29/2008 13:44'!
removeSubclass: aClassEditor
	removals add: aClassEditor.
	additions remove: aClassEditor ifAbsent: [].! !
PureBehaviorEditor initialize!
